CREATE OR REPLACE PACKAGE pkg_consultes_dwh
IS
    RSP logs.RSP%TYPE;

    -- El nmero total de jugadors en actiu en tots els gneres i modalitats.
    PROCEDURE lst_dwh_totalJugadors(
        RSP OUT logs.RSP%TYPE
    );

    -- Donada una competici, el seu mxim anotador en la temporada en curs
    PROCEDURE lst_dwh_anotadorsCompeticio(
        p_idCompeticio dwh_anotadorsCompeticio.idCompeticio%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- El jugador ms ben pagat de cada modalitat i gnere.
    PROCEDURE lst_dwh_millorPagats(
        RSP OUT logs.RSP%TYPE
    );

    -- El jugador amb ms guanys acumulats al llarg de tota la seva carrera esportiva.
    PROCEDURE lst_dwh_totalGuanys(
        RSP OUT logs.RSP%TYPE
    );

    -- Donat un pas, una temporada, un gnere i una modalitat, el sou mig anual dels jugadors.
    PROCEDURE lst_dwh_souMigJugadors(
        p_idPais IN dwh_souMigJugadors.idPais%TYPE,
        p_anyTemporada IN dwh_souMigJugadors.anyTemporada%TYPE,
        p_genere IN dwh_souMigJugadors.genere%TYPE,
        p_modalitat IN dwh_souMigJugadors.modalitat%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Donada una competici i una temporada, els millors equips ofensius i defensius.
    PROCEDURE lst_dwh_millorsEquips(
        p_idCompeticio dwh_millorsEquips.idCompeticio%TYPE,
        p_anyTemporada dwh_millorsEquips.anyTemporada%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Per a cada gnere i modalitat, el millor jugador del mn en el darrer any.
    PROCEDURE lst_dwh_millorJugador(
        RSP OUT logs.RSP%TYPE
    );

END pkg_consultes_dwh;
/
CREATE OR REPLACE PACKAGE BODY pkg_consultes_dwh
IS
    -- El nmero total de jugadors en actiu en tots els gneres i modalitats.
    PROCEDURE lst_dwh_totalJugadors(
        RSP OUT logs.RSP%TYPE
    )
    IS
        m_numJugadors dwh_totalJugadors.total%TYPE;

    BEGIN

        SELECT total INTO m_numJugadors FROM dwh_totalJugadors;

        DBMS_OUTPUT.PUT_LINE('==============');
        DBMS_OUTPUT.PUT_LINE('Total jugadors');
        DBMS_OUTPUT.PUT_LINE('==============');

        DBMS_OUTPUT.PUT_LINE(m_numJugadors);

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, RSP)
        VALUES ('lst_dwh_totalJugadors', SYSDATE, RSP);

    EXCEPTION
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en la consulta ' || SQLERRM;
            INSERT INTO logs (nomProc, data, RSP)
            VALUES ('lst_dwh_totalJugadors', SYSDATE, RSP);

    END lst_dwh_totalJugadors;


    -- Donada una competici, el seu mxim anotador en la temporada en curs
    PROCEDURE lst_dwh_anotadorsCompeticio(
        p_idCompeticio dwh_anotadorsCompeticio.idCompeticio%TYPE,
        RSP OUT logs.RSP%TYPE
    )
    IS

        e_competicioNF EXCEPTION;
        trobat NUMBER;
        m_nom dwh_anotadorsCompeticio.nom%TYPE;
        m_cognoms dwh_anotadorsCompeticio.cognoms%TYPE;
        m_punts dwh_anotadorsCompeticio.punts%TYPE;
    BEGIN
        -- Control ID competicio existent
        SELECT COUNT(*) INTO trobat FROM competicions WHERE idCompeticio = p_idCompeticio;
        IF (trobat = 0) THEN
            RAISE e_competicioNF;
        END IF;

        DBMS_OUTPUT.PUT_LINE('====================');
        DBMS_OUTPUT.PUT_LINE('Jugador ms anotador');
        DBMS_OUTPUT.PUT_LINE('====================');

        SELECT nom, cognoms, punts
          INTO m_nom, m_cognoms, m_punts
          FROM (SELECT nom, cognoms, punts
                  FROM dwh_anotadorsCompeticio
                 WHERE idCompeticio = p_idCompeticio
                   AND anyTemporada = EXTRACT(YEAR FROM SYSDATE)
                   ORDER BY punts DESC)
         WHERE rownum < 2;

        DBMS_OUTPUT.PUT_LINE(m_nom || ', ' ||  m_cognoms || ', ' || m_punts);

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('lst_dwh_anotadorsCompeticio', SYSDATE, p_idCompeticio, RSP);

    EXCEPTION
        WHEN e_competicioNF THEN
            RSP := 'ERROR: Competicio [' || p_idCompeticio || '] no trobada';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_dwh_anotadorsCompeticio', SYSDATE, p_idCompeticio, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en la consulta ' || SQLERRM;
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_dwh_anotadorsCompeticio', SYSDATE, p_idCompeticio, RSP);

    END lst_dwh_anotadorsCompeticio;


    -- El jugador ms ben pagat de cada modalitat i gnere.
    PROCEDURE lst_dwh_millorPagats(
        RSP OUT logs.RSP%TYPE
    )
    IS
        CURSOR c_millorPagats IS
            SELECT nom, cognoms, modalitat, genere, salari
              FROM (SELECT nom, cognoms, modalitat, genere, salari
                      FROM dwh_millorPagats
                     WHERE modalitat = 'N'
                       AND genere = 'M'
                     ORDER BY salari DESC)
             WHERE rownum < 2
            UNION
            SELECT nom, cognoms, modalitat, genere, salari
              FROM (SELECT nom, cognoms, modalitat, genere, salari
                      FROM dwh_millorPagats
                     WHERE modalitat = 'N'
                       AND genere = 'F'
                     ORDER BY salari DESC)
             WHERE rownum < 2
            UNION
            SELECT nom, cognoms, modalitat, genere, salari
              FROM (SELECT nom, cognoms, modalitat, genere, salari
                      FROM dwh_millorPagats
                     WHERE modalitat = 'C'
                       AND genere = 'M'
                     ORDER BY salari DESC)
             WHERE rownum < 2
            UNION
            SELECT nom, cognoms, modalitat, genere, salari
              FROM (SELECT nom, cognoms, modalitat, genere, salari
                      FROM dwh_millorPagats
                     WHERE modalitat = 'C'
                       AND genere = 'F'
                     ORDER BY salari DESC)
             WHERE rownum < 2;

        m_nom dwh_millorPagats.nom%TYPE;
        m_cognoms dwh_millorPagats.cognoms%TYPE;
        m_modalitat dwh_millorPagats.modalitat%TYPE;
        m_genere dwh_millorPagats.genere%TYPE;
        m_salari dwh_millorPagats.salari%TYPE;

    BEGIN

        DBMS_OUTPUT.PUT_LINE('======================');
        DBMS_OUTPUT.PUT_LINE('Jugadors millor pagats');
        DBMS_OUTPUT.PUT_LINE('======================');

        OPEN c_millorPagats;
        -- Mostrem les dades de cada jugador
        LOOP
            FETCH c_millorPagats INTO m_nom, m_cognoms, m_modalitat, m_genere, m_salari;
            EXIT WHEN c_millorPagats%NOTFOUND;

            DBMS_OUTPUT.PUT_LINE(m_nom || ', ' ||  m_cognoms || ', ' || m_modalitat || ', ' || m_genere || ', ' || m_salari);
        END LOOP;

        CLOSE c_millorPagats;

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, RSP)
        VALUES ('lst_dwh_millorPagats', SYSDATE, RSP);

    EXCEPTION
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en la consulta ' || SQLERRM;
            INSERT INTO logs (nomProc, data, RSP)
            VALUES ('lst_dwh_millorPagats', SYSDATE, RSP);


    END lst_dwh_millorPagats;

    -- El jugador amb ms guanys acumulats al llarg de tota la seva carrera esportiva.
    PROCEDURE lst_dwh_totalGuanys(
        RSP OUT logs.RSP%TYPE
    )
    IS
        m_nom dwh_totalGuanys.nom%TYPE;
        m_cognoms dwh_totalGuanys.cognoms%TYPE;
        m_guanys dwh_totalGuanys.guanys%TYPE;

    BEGIN

        SELECT nom, cognoms, guanys
          INTO m_nom, m_cognoms, m_guanys
          FROM (SELECT nom, cognoms, guanys FROM dwh_totalGuanys ORDER BY guanys DESC)
         WHERE rownum < 2;

        DBMS_OUTPUT.PUT_LINE('======================');
        DBMS_OUTPUT.PUT_LINE('Jugador amb ms guanys');
        DBMS_OUTPUT.PUT_LINE('======================');

        DBMS_OUTPUT.PUT_LINE(m_nom || ', ' || m_cognoms || ', ' || m_guanys);

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, RSP)
        VALUES ('lst_dwh_totalGuanys', SYSDATE, RSP);

    EXCEPTION
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en la consulta ' || SQLERRM;
            INSERT INTO logs (nomProc, data, RSP)
            VALUES ('lst_dwh_totalGuanys', SYSDATE, RSP);

    END lst_dwh_totalGuanys;

    -- Donat un pas, una temporada, un gnere i una modalitat, el sou mig anual dels jugadors.
    PROCEDURE lst_dwh_souMigJugadors(
        p_idPais IN dwh_souMigJugadors.idPais%TYPE,
        p_anyTemporada IN dwh_souMigJugadors.anyTemporada%TYPE,
        p_genere IN dwh_souMigJugadors.genere%TYPE,
        p_modalitat IN dwh_souMigJugadors.modalitat%TYPE,
        RSP OUT logs.RSP%TYPE
    )
    IS
        e_paisNF EXCEPTION;
        e_genereCHK EXCEPTION;
        e_modalitatCHK EXCEPTION;
        m_souMig dwh_souMigJugadors.souMig%TYPE;
        trobat NUMBER;

    BEGIN
        -- Control pais existent
        SELECT COUNT(*) INTO trobat FROM paisos WHERE idPais = p_idPais;
        IF (trobat = 0) THEN
            RAISE e_paisNF;
        END IF;

        -- Control genere
        IF (p_genere NOT IN ('M','F')) THEN
            RAISE e_genereCHK;
        END IF;

        -- Control modalitat
        IF (p_modalitat NOT IN ('N','C')) THEN
            RAISE e_modalitatCHK;
        END IF;

        SELECT souMig
          INTO m_souMig
          FROM dwh_souMigJugadors
         WHERE idPais = p_idPais
           AND anyTemporada = p_anyTemporada
           AND genere = p_genere
           AND modalitat = p_modalitat;

        DBMS_OUTPUT.PUT_LINE('=====================');
        DBMS_OUTPUT.PUT_LINE('Sou mig dels jugadors');
        DBMS_OUTPUT.PUT_LINE('=====================');

        DBMS_OUTPUT.PUT_LINE(m_souMig);

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('lst_dwh_souMigJugadors', SYSDATE, p_idPais || ', ' || p_anyTemporada || ', ' || p_genere || ', ' || p_modalitat, RSP);

    EXCEPTION
        WHEN e_paisNF THEN
            RSP := 'ERROR: Codi de pais [' || p_idPais || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_dwh_souMigJugadors', SYSDATE, p_idPais || ', ' || p_anyTemporada || ', ' || p_genere || ', ' || p_modalitat, RSP);
        WHEN e_genereCHK THEN
            RSP := 'ERROR: Genere [' || p_genere || '] no vlid';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_dwh_souMigJugadors', SYSDATE, p_idPais || ', ' || p_anyTemporada || ', ' || p_genere || ', ' || p_modalitat, RSP);
        WHEN e_modalitatCHK THEN
            RSP := 'ERROR: Modalitat [' || p_modalitat || '] no vlida';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_dwh_souMigJugadors', SYSDATE, p_idPais || ', ' || p_anyTemporada || ', ' || p_genere || ', ' || p_modalitat, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en la consulta ' || SQLERRM;
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_dwh_souMigJugadors', SYSDATE, p_idPais || ', ' || p_anyTemporada || ', ' || p_genere || ', ' || p_modalitat, RSP);

    END lst_dwh_souMigJugadors;

    -- Donada una competici i una temporada, els millors equips ofensius i defensius.
    PROCEDURE lst_dwh_millorsEquips(
        p_idCompeticio dwh_millorsEquips.idCompeticio%TYPE,
        p_anyTemporada dwh_millorsEquips.anyTemporada%TYPE,
        RSP OUT logs.RSP%TYPE
    )
    IS
        CURSOR c_millorsEquips IS
            SELECT tipus, nomEquip, punts
              FROM (SELECT tipus, nomEquip, punts
                      FROM dwh_millorsEquips
                     WHERE idCompeticio = p_idCompeticio
                       AND anyTemporada = p_anyTemporada
                       AND tipus = 'O'
                     ORDER BY punts DESC)
             WHERE rownum < 2
             UNION
            SELECT tipus, nomEquip, punts
              FROM (SELECT tipus, nomEquip, punts
                      FROM dwh_millorsEquips
                     WHERE idCompeticio = p_idCompeticio
                       AND anyTemporada = p_anyTemporada
                       AND tipus = 'D'
                     ORDER BY punts)
             WHERE rownum < 2;

        e_competicioNF EXCEPTION;
        trobat NUMBER;
        m_tipus dwh_millorsEquips.tipus%TYPE;
        m_nomEquip dwh_millorsEquips.nomEquip%TYPE;
        m_punts dwh_millorsEquips.punts%TYPE;

    BEGIN
        -- Control ID competicio existent
        SELECT COUNT(*) INTO trobat FROM competicions WHERE idCompeticio = p_idCompeticio;
        IF (trobat = 0) THEN
            RAISE e_competicioNF;
        END IF;

        DBMS_OUTPUT.PUT_LINE('==============');
        DBMS_OUTPUT.PUT_LINE('Millors equips');
        DBMS_OUTPUT.PUT_LINE('==============');

        OPEN c_millorsEquips;
        -- Mostrem les dades de cada jugador
        LOOP
            FETCH c_millorsEquips INTO m_tipus, m_nomEquip, m_punts;
            EXIT WHEN c_millorsEquips%NOTFOUND;

            DBMS_OUTPUT.PUT_LINE(m_tipus || ', ' || m_nomEquip || ', ' || m_punts);
        END LOOP;

        CLOSE c_millorsEquips;

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('lst_dwh_millorsEquips', SYSDATE, p_idCompeticio || ', ' || p_anyTemporada, RSP);

    EXCEPTION
        WHEN e_competicioNF THEN
            RSP := 'ERROR: Competicio [' || p_idCompeticio || '] no trobada';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_dwh_millorsEquips', SYSDATE, p_idCompeticio || ', ' || p_anyTemporada, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en la consulta ' || SQLERRM;
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('lst_dwh_millorsEquips', SYSDATE, p_idCompeticio || ', ' || p_anyTemporada, RSP);

    END lst_dwh_millorsEquips;


    -- Per a cada gnere i modalitat, el millor jugador del mn en el darrer any.
    PROCEDURE lst_dwh_millorJugador(
        RSP OUT logs.RSP%TYPE
    )
    IS

        CURSOR c_millorJugador IS
            SELECT genere, modalitat, nom, cognoms, valoracioP
              FROM (SELECT genere, modalitat, nom, cognoms, valoracioP
                      FROM dwh_millorJugador
                     WHERE modalitat = 'N'
                       AND genere = 'M'
                       AND anyTemporada = EXTRACT(YEAR FROM SYSDATE)
                     ORDER BY valoracioP DESC)
             WHERE rownum < 2
            UNION
            SELECT genere, modalitat, nom, cognoms, valoracioP
              FROM (SELECT genere, modalitat, nom, cognoms, valoracioP
                      FROM dwh_millorJugador
                     WHERE modalitat = 'N'
                       AND genere = 'F'
                       AND anyTemporada = EXTRACT(YEAR FROM SYSDATE)
                     ORDER BY valoracioP DESC)
             WHERE rownum < 2
            UNION
            SELECT genere, modalitat, nom, cognoms, valoracioP
              FROM (SELECT genere, modalitat, nom, cognoms, valoracioP
                      FROM dwh_millorJugador
                     WHERE modalitat = 'C'
                       AND genere = 'M'
                       AND anyTemporada = EXTRACT(YEAR FROM SYSDATE)
                     ORDER BY valoracioP DESC)
             WHERE rownum < 2
            UNION
            SELECT genere, modalitat, nom, cognoms, valoracioP
              FROM (SELECT genere, modalitat, nom, cognoms, valoracioP
                      FROM dwh_millorJugador
                     WHERE modalitat = 'C'
                       AND genere = 'F'
                       AND anyTemporada = EXTRACT(YEAR FROM SYSDATE)
                     ORDER BY valoracioP DESC)
             WHERE rownum < 2;

        m_genere dwh_millorJugador.genere%TYPE;
        m_modalitat dwh_millorJugador.modalitat%TYPE;
        m_nom dwh_millorJugador.nom%TYPE;
        m_cognoms dwh_millorJugador.cognoms%TYPE;
        m_valoracioP dwh_millorJugador.valoracioP%TYPE;

    BEGIN
        DBMS_OUTPUT.PUT_LINE('================');
        DBMS_OUTPUT.PUT_LINE('Millors jugadors');
        DBMS_OUTPUT.PUT_LINE('================');

        OPEN c_millorJugador;
        -- Mostrem les dades de cada jugador
        LOOP
            FETCH c_millorJugador INTO m_genere, m_modalitat, m_nom, m_cognoms, m_valoracioP;
            EXIT WHEN c_millorJugador%NOTFOUND;

            DBMS_OUTPUT.PUT_LINE(m_genere || ', ' || m_modalitat || ', ' || m_nom || ', ' ||  m_cognoms || ', ' || m_valoracioP);
        END LOOP;

        CLOSE c_millorJugador;

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, RSP)
        VALUES ('lst_dwh_millorJugador', SYSDATE, RSP);

    EXCEPTION
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en la consulta ' || SQLERRM;
            INSERT INTO logs (nomProc, data, RSP)
            VALUES ('lst_dwh_millorJugador', SYSDATE, RSP);

    END lst_dwh_millorJugador;

END pkg_consultes_dwh;