﻿CREATE OR REPLACE PACKAGE pkg_gestioJugadors
IS
    RSP logs.RSP%TYPE;

    -- Dona d'alta un jugador en el sistema. Ja existeix com a persona.
    PROCEDURE ges_altaJugador(
        p_idPersona IN persones.idPersona%TYPE,
        p_idFederacio IN federacions.idFederacio%TYPE,
        p_numFederat IN jugadors.numFederat%TYPE,
        p_genere IN jugadors.genere%TYPE,
        p_modalitat IN jugadors.modalitat%TYPE,
        p_alcada IN jugadors.alcada%TYPE,
        p_pes IN jugadors.pes%TYPE,
        p_posicio IN jugadors.posicio%TYPE,
        p_urlWeb IN jugadors.urlWeb%TYPE,
        p_urlVideo IN jugadors.urlVideo%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Dona d'alta un jugador i una persona en el sistema. Cas persona i jugador nous.
    PROCEDURE ges_altaJugadorPersona(
        p_nom IN persones.nom%TYPE,
        p_cognoms IN persones.cognoms%TYPE,
        p_idPais IN persones.idPais%TYPE,
        p_dataNaixement IN persones.dataNaixement%TYPE,
        p_idFederacio IN federacions.idFederacio%TYPE,
        p_numFederat IN jugadors.numFederat%TYPE,
        p_genere IN jugadors.genere%TYPE,
        p_modalitat IN jugadors.modalitat%TYPE,
        p_alcada IN jugadors.alcada%TYPE,
        p_pes IN jugadors.pes%TYPE,
        p_posicio IN jugadors.posicio%TYPE,
        p_urlWeb IN jugadors.urlWeb%TYPE,
        p_urlVideo IN jugadors.urlVideo%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Dona de baixa un jugador. Baixa per malaltia/lesió o motius personals.
    PROCEDURE ges_baixaJugador(
        p_idJugador IN jugadors.idJugador%TYPE,
        p_tipus subestats.tipus%TYPE,
        p_descripcio subestats.descripcio%TYPE,
        p_dataDisponibilitat subestats.dataDisponibilitat%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Dona d'alta un jugador que esta de baixa.
    PROCEDURE ges_reactivaJugador(
        p_idJugador IN jugadors.idJugador%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Dona de baixa un jugador. Retirat com a jugador.
    PROCEDURE ges_retiraJugador(
        p_idJugador IN jugadors.idJugador%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Elimina un jugador de la BBDD
    PROCEDURE ges_esborraJugador(
        p_idJugador IN jugadors.idJugador%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Permet modificar les dades d'un jugador
    PROCEDURE ges_modificaJugador(
        p_idJugador IN jugadors.idJugador%TYPE,
        p_modalitat IN jugadors.modalitat%TYPE,
        p_alcada IN jugadors.alcada%TYPE,
        p_pes IN jugadors.pes%TYPE,
        p_posicio IN jugadors.posicio%TYPE,
        p_urlWeb IN jugadors.urlWeb%TYPE,
        p_urlVideo IN jugadors.urlVideo%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Dona d'alta o modifica les estaditiques per partit d'un jugador
    PROCEDURE ges_creaEstadistiques(
        p_idPartit IN partits.idPartit%TYPE,
        p_idJugador IN jugadors.idJugador%TYPE,
        p_mins IN convocats.mins%TYPE,
        p_t1I IN convocats.t1I%TYPE,
        p_t1E IN convocats.t1E%TYPE,
        p_t2I IN convocats.t2I%TYPE,
        p_t2E IN convocats.t2E%TYPE,
        p_t3I IN convocats.t3I%TYPE,
        p_t3E IN convocats.t3E%TYPE,
        p_rd IN convocats.rd%TYPE,
        p_ro IN convocats.ro%TYPE,
        p_asi IN convocats.asi%TYPE,
        p_tf IN convocats.tf%TYPE,
        p_tc IN convocats.tc%TYPE,
        p_pr IN convocats.pr%TYPE,
        p_pp IN convocats.pp%TYPE,
        p_fc IN convocats.fc%TYPE,
        p_fr IN convocats.fr%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Elimina una estadística d'un partit - jugador
    PROCEDURE ges_esborraEstadistica(
        p_idPartit IN partits.idPartit%TYPE,
        p_idJugador IN jugadors.idJugador%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Genera estadistiques fictices per a un jugardor - partit
    PROCEDURE ges_generaEstadistiques(
        p_idPartit IN partits.idPartit%TYPE,
        p_idJugador IN jugadors.idJugador%TYPE,
        RSP OUT logs.RSP%TYPE
    );

END pkg_gestioJugadors;
/
CREATE OR REPLACE PACKAGE BODY pkg_gestioJugadors
IS

    -- Dona d'alta un jugador en el sistema. Ja existeix com a persona.
    PROCEDURE ges_altaJugador(
        p_idPersona IN persones.idPersona%TYPE,
        p_idFederacio IN federacions.idFederacio%TYPE,
        p_numFederat IN jugadors.numFederat%TYPE,
        p_genere IN jugadors.genere%TYPE,
        p_modalitat IN jugadors.modalitat%TYPE,
        p_alcada IN jugadors.alcada%TYPE,
        p_pes IN jugadors.pes%TYPE,
        p_posicio IN jugadors.posicio%TYPE,
        p_urlWeb IN jugadors.urlWeb%TYPE,
        p_urlVideo IN jugadors.urlVideo%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_personaNF EXCEPTION;
        e_federacioNF EXCEPTION;
        e_genereCHK EXCEPTION;
        e_modalitatCHK EXCEPTION;
        e_posicioCHK EXCEPTION;
        e_null EXCEPTION;

        trobat NUMBER;

    BEGIN

        -- Control ID persona existent
        SELECT COUNT(*) INTO trobat FROM persones WHERE idPersona = p_idPersona;
        IF (trobat = 0) THEN
            RAISE e_personaNF;
        END IF;

        -- Control ID federacio existent
        SELECT COUNT(*) INTO trobat FROM federacions WHERE idFederacio = p_idFederacio;
        IF (trobat = 0) THEN
            RAISE e_federacioNF;
        END IF;

        -- Control genere
        IF (p_genere NOT IN ('M','F')) THEN
            RAISE e_genereCHK;
        END IF;

        -- Control modalitat
        IF (p_modalitat NOT IN ('N','C')) THEN
            RAISE e_modalitatCHK;
        END IF;

        -- Control posicio
        IF (p_posicio NOT IN (1,2,3,4,5)) THEN
            RAISE e_posicioCHK;
        END IF;
        
        -- Control el numero de federat es obligatori
        IF (p_numFederat IS NULL) THEN
            RAISE e_null;
        END IF;

        -- Si no hi ha cap error, podem insertar el jugador
        INSERT INTO jugadors (idPersona, idFederacio, numFederat, genere,
                              modalitat, alcada, pes, posicio, urlWeb, urlVideo, estat)
        VALUES (p_idPersona, p_idFederacio, p_numFederat, p_genere,
                p_modalitat, p_alcada, p_pes, p_posicio, p_urlWeb, p_urlVideo, 'A');

        -- Actualitza el nombre de jugadors de dwh_totalJugadors
        UPDATE dwh_totalJugadors SET total = (SELECT COUNT(*) FROM jugadors WHERE estat = 'A');

        DBMS_OUTPUT.PUT_LINE('Jugador creat');

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_altaJugador', SYSDATE,
                p_idPersona || ', ' || p_idFederacio || ', ' || p_numFederat || ', ' ||
                p_genere || ', ' || p_modalitat || ', ' || p_alcada || ', ' ||
                p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);

    EXCEPTION
        WHEN e_personaNF THEN
            RSP := 'ERROR: ID de persona [' || p_idPersona || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugador', SYSDATE,
                     p_idPersona || ', ' || p_idFederacio || ', ' || p_numFederat || ', ' ||
                     p_genere || ', ' || p_modalitat || ', ' || p_alcada || ', ' ||
                     p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);
        WHEN e_federacioNF THEN
            RSP := 'ERROR: ID de federacio [' || p_idFederacio || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugador', SYSDATE,
                     p_idPersona || ', ' || p_idFederacio || ', ' || p_numFederat || ', ' ||
                     p_genere || ', ' || p_modalitat || ', ' || p_alcada || ', ' ||
                     p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);
        WHEN e_genereCHK THEN
            RSP := 'ERROR: Genere [' || p_genere || '] no vàlid';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugador', SYSDATE,
                     p_idPersona || ', ' || p_idFederacio || ', ' || p_numFederat || ', ' ||
                     p_genere || ', ' || p_modalitat || ', ' || p_alcada || ', ' ||
                     p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);
        WHEN e_modalitatCHK THEN
            RSP := 'ERROR: Modalitat [' || p_modalitat || '] no vàlida';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugador', SYSDATE,
                     p_idPersona || ', ' || p_idFederacio || ', ' || p_numFederat || ', ' ||
                     p_genere || ', ' || p_modalitat || ', ' || p_alcada || ', ' ||
                     p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);
        WHEN e_posicioCHK THEN
            RSP := 'ERROR: Codi de posició [' || p_posicio || '] no vàlid';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugador', SYSDATE,
                     p_idPersona || ', ' || p_idFederacio || ', ' || p_numFederat || ', ' ||
                     p_genere || ', ' || p_modalitat || ', ' || p_alcada || ', ' ||
                     p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);
        WHEN e_null THEN
            RSP := 'ERROR: El numero de federat és obligatori';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugador', SYSDATE,
                     p_idPersona || ', ' || p_idFederacio || ', ' || p_numFederat || ', ' ||
                     p_genere || ', ' || p_modalitat || ', ' || p_alcada || ', ' ||
                     p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en alta de jugador ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugador', SYSDATE,
                     p_idPersona || ', ' || p_idFederacio || ', ' || p_numFederat || ', ' ||
                     p_genere || ', ' || p_modalitat || ', ' || p_alcada || ', ' ||
                     p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);

    END ges_altaJugador;

    -- Dona d'alta un jugador i una persona en el sistema. Cas persona i jugador nous.
    PROCEDURE ges_altaJugadorPersona(
        p_nom IN persones.nom%TYPE,
        p_cognoms IN persones.cognoms%TYPE,
        p_idPais IN persones.idPais%TYPE,
        p_dataNaixement IN persones.dataNaixement%TYPE,
        p_idFederacio IN federacions.idFederacio%TYPE,
        p_numFederat IN jugadors.numFederat%TYPE,
        p_genere IN jugadors.genere%TYPE,
        p_modalitat IN jugadors.modalitat%TYPE,
        p_alcada IN jugadors.alcada%TYPE,
        p_pes IN jugadors.pes%TYPE,
        p_posicio IN jugadors.posicio%TYPE,
        p_urlWeb IN jugadors.urlWeb%TYPE,
        p_urlVideo IN jugadors.urlVideo%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_paisNF EXCEPTION;
        e_federacioNF EXCEPTION;
        e_genereCHK EXCEPTION;
        e_modalitatCHK EXCEPTION;
        e_posicioCHK EXCEPTION;
        e_null EXCEPTION;
        trobat NUMBER;

    BEGIN

        -- Control pais existent
        SELECT COUNT(*) INTO trobat FROM paisos WHERE idPais = p_idPais;
        IF (trobat = 0) THEN
            RAISE e_paisNF;
        END IF;

        -- Control ID federacio existent
        SELECT COUNT(*) INTO trobat FROM federacions WHERE idFederacio = p_idFederacio;
        IF (trobat = 0) THEN
            RAISE e_federacioNF;
        END IF;

        -- Control genere
        IF (p_genere NOT IN ('M','F')) THEN
            RAISE e_genereCHK;
        END IF;

        -- Control modalitat
        IF (p_modalitat NOT IN ('N','C')) THEN
            RAISE e_modalitatCHK;
        END IF;

        -- Control posicio
        IF (p_posicio NOT IN (1,2,3,4,5)) THEN
            RAISE e_posicioCHK;
        END IF;

        -- Control camps obligatoris
        IF (p_numFederat IS NULL OR p_nom IS NULL OR p_cognoms IS NULL OR p_dataNaixement IS NULL) THEN
            RAISE e_null;
        END IF;

        -- Si no hi ha cap error, podem insertar la persona
        INSERT INTO persones (idPais, nom, cognoms, dataNaixement, estat)
        VALUES (p_idPais, p_nom, p_cognoms, p_dataNaixement, 'A');

        -- Si no hi ha cap error, podem insertar el jugador
        INSERT INTO jugadors (idPersona, idFederacio, numFederat, genere,
                              modalitat, alcada, pes, posicio, urlWeb, urlVideo, estat)
        VALUES (s_persones1.CURRVAL, p_idFederacio, p_numFederat, p_genere,
                p_modalitat, p_alcada, p_pes, p_posicio, p_urlWeb, p_urlVideo, 'A');

        -- Actualitza el nombre de jugadors de dwh_totalJugadors
        UPDATE dwh_totalJugadors SET total = (SELECT COUNT(*) FROM jugadors WHERE estat = 'A');

        DBMS_OUTPUT.PUT_LINE('Persona creada');
        DBMS_OUTPUT.PUT_LINE('Jugador creat');

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_altaJugadorPersona', SYSDATE,
                p_nom || ', ' || p_cognoms || ', ' || p_idPais || ', ' || p_dataNaixement || ', ' ||
                p_idFederacio || ', ' || p_numFederat || ', ' || p_genere || ', ' || p_modalitat || ', ' ||
                p_alcada || ', ' || p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);

    EXCEPTION
        WHEN e_paisNF THEN
            RSP := 'ERROR: Codi de pais [' || p_idPais || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugadorPersona', SYSDATE,
                     p_nom || ', ' || p_cognoms || ', ' || p_idPais || ', ' || p_dataNaixement || ', ' ||
                     p_idFederacio || ', ' || p_numFederat || ', ' || p_genere || ', ' || p_modalitat || ', ' ||
                     p_alcada || ', ' || p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);
        WHEN e_federacioNF THEN
            RSP := 'ERROR: ID de federacio [' || p_idFederacio || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugadorPersona', SYSDATE,
                     p_nom || ', ' || p_cognoms || ', ' || p_idPais || ', ' || p_dataNaixement || ', ' ||
                     p_idFederacio || ', ' || p_numFederat || ', ' || p_genere || ', ' || p_modalitat || ', ' ||
                     p_alcada || ', ' || p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);
        WHEN e_genereCHK THEN
            RSP := 'ERROR: Genere [' || p_genere || '] no vàlid';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugadorPersona', SYSDATE,
                     p_nom || ', ' || p_cognoms || ', ' || p_idPais || ', ' || p_dataNaixement || ', ' ||
                     p_idFederacio || ', ' || p_numFederat || ', ' || p_genere || ', ' || p_modalitat || ', ' ||
                     p_alcada || ', ' || p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);
        WHEN e_modalitatCHK THEN
            RSP := 'ERROR: Modalitat [' || p_modalitat || '] no vàlida';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugadorPersona', SYSDATE,
                     p_nom || ', ' || p_cognoms || ', ' || p_idPais || ', ' || p_dataNaixement || ', ' ||
                     p_idFederacio || ', ' || p_numFederat || ', ' || p_genere || ', ' || p_modalitat || ', ' ||
                     p_alcada || ', ' || p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);
        WHEN e_posicioCHK THEN
            RSP := 'ERROR: Codi de posició [' || p_posicio || '] no vàlid';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugadorPersona', SYSDATE,
                     p_nom || ', ' || p_cognoms || ', ' || p_idPais || ', ' || p_dataNaixement || ', ' ||
                     p_idFederacio || ', ' || p_numFederat || ', ' || p_genere || ', ' || p_modalitat || ', ' ||
                     p_alcada || ', ' || p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);
        WHEN e_null THEN
            RSP := 'ERROR: El numero de federat és obligatori';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugadorPersona', SYSDATE,
                     p_nom || ', ' || p_cognoms || ', ' || p_idPais || ', ' || p_dataNaixement || ', ' ||
                     p_idFederacio || ', ' || p_numFederat || ', ' || p_genere || ', ' || p_modalitat || ', ' ||
                     p_alcada || ', ' || p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en alta de jugador ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaJugadorPersona', SYSDATE,
                     p_nom || ', ' || p_cognoms || ', ' || p_idPais || ', ' || p_dataNaixement || ', ' ||
                     p_idFederacio || ', ' || p_numFederat || ', ' || p_genere || ', ' || p_modalitat || ', ' ||
                     p_alcada || ', ' || p_pes || ', ' || p_posicio || ', ' || p_urlWeb || ', ' || p_urlVideo, RSP);

    END ges_altaJugadorPersona;

    -- Dona de baixa un jugador. Baixa per malaltia/lesió.
    PROCEDURE ges_baixaJugador(
        p_idJugador IN jugadors.idJugador%TYPE,
        p_tipus subestats.tipus%TYPE,
        p_descripcio subestats.descripcio%TYPE,
        p_dataDisponibilitat subestats.dataDisponibilitat%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_jugadorNF EXCEPTION;
        e_jugadorBaixa EXCEPTION;
        e_tipusCHK EXCEPTION;
        trobat NUMBER;
        m_estatActual jugadors.estat%TYPE;

    BEGIN

        -- Control ID jugador existent
        SELECT COUNT(*) INTO trobat FROM jugadors WHERE idJugador = p_idJugador;
        IF (trobat = 0) THEN
            RAISE e_jugadorNF;
        END IF;

        -- Control jugador de baixa
        SELECT estat INTO m_estatActual FROM jugadors WHERE idJugador = p_idJugador;
        IF (m_estatActual = 'B') THEN
            RAISE e_jugadorBaixa;
        END IF;

        -- Control tipus de baixa
        IF (p_tipus NOT IN ('P','M')) THEN
            RAISE e_tipusCHK;
        END IF;

        -- Marquem de baixa el jugador
        UPDATE jugadors SET estat = 'B' WHERE idJugador = p_idJugador;

        -- Crear nova entrada de subestat
        INSERT INTO subestats (idJugador, tipus, descripcio, dataDisponibilitat, dataInici)
        VALUES (p_idJugador, p_tipus, p_descripcio, p_dataDisponibilitat, SYSDATE);

        -- Actualitza el nombre de jugadors de dwh_totalJugadors
        UPDATE dwh_totalJugadors SET total = (SELECT COUNT(*) FROM jugadors WHERE estat = 'A');

        DBMS_OUTPUT.PUT_LINE('Jugador [' || p_idJugador || '] donat de baixa');

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_baixaJugador', SYSDATE, p_idJugador || ', ' || p_tipus || ', ' || p_descripcio || ', ' || p_dataDisponibilitat, RSP);

    EXCEPTION
        WHEN e_jugadorNF THEN
            RSP := 'ERROR: ID de jugador [' || p_idJugador || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_baixaJugador', SYSDATE, p_idJugador || ', ' || p_tipus || ', ' || p_descripcio || ', ' || p_dataDisponibilitat, RSP);
        WHEN e_jugadorBaixa THEN
            RSP := 'ERROR: El jugador [' || p_idJugador || '] ja es troba de baixa';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_baixaJugador', SYSDATE, p_idJugador || ', ' || p_tipus || ', ' || p_descripcio || ', ' || p_dataDisponibilitat, RSP);
        WHEN e_tipusCHK THEN
            RSP := 'ERROR: Tipus de baixa [' || p_tipus || '] no reconegut';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_baixaJugador', SYSDATE, p_idJugador || ', ' || p_tipus || ', ' || p_descripcio || ', ' || p_dataDisponibilitat, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en baixa de jugador ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_baixaJugador', SYSDATE, p_idJugador || ', ' || p_tipus || ', ' || p_descripcio || ', ' || p_dataDisponibilitat, RSP);

    END ges_baixaJugador;

    -- Dona d'alta un jugador que esta de baixa.
    PROCEDURE ges_reactivaJugador(
        p_idJugador IN jugadors.idJugador%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_jugadorNF EXCEPTION;
        e_jugadorAlta EXCEPTION;
        trobat NUMBER;
        m_estatActual jugadors.estat%TYPE;

    BEGIN

        -- Control ID jugador existent
        SELECT COUNT(*) INTO trobat FROM jugadors WHERE idJugador = p_idJugador;
        IF (trobat = 0) THEN
            RAISE e_jugadorNF;
        END IF;

        -- Control jugador d'alta
        SELECT estat INTO m_estatActual FROM jugadors WHERE idJugador = p_idJugador;
        IF (m_estatActual = 'A') THEN
            RAISE e_jugadorAlta;
        END IF;

        -- Marquem d'alta el jugador
        UPDATE jugadors SET estat = 'A' WHERE idJugador = p_idJugador;

        -- Actualitza el subestat
        UPDATE subestats SET dataFiReal = SYSDATE WHERE idJugador = p_idJugador AND dataFiReal IS NULL;

        -- Actualitza el nombre de jugadors de dwh_totalJugadors
        UPDATE dwh_totalJugadors SET total = (SELECT COUNT(*) FROM jugadors WHERE estat = 'A');

        DBMS_OUTPUT.PUT_LINE('Jugador [' || p_idJugador || '] donat de alta');

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_reactivaJugador', SYSDATE, p_idJugador, RSP);

    EXCEPTION
        WHEN e_jugadorNF THEN
            RSP := 'ERROR: ID de jugador [' || p_idJugador || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_reactivaJugador', SYSDATE, p_idJugador, RSP);
        WHEN e_jugadorAlta THEN
            RSP := 'ERROR: El jugador [' || p_idJugador || '] ja es troba en alta';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_reactivaJugador', SYSDATE, p_idJugador, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en baixa de jugador ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_reactivaJugador', SYSDATE, p_idJugador, RSP);

    END ges_reactivaJugador;

    -- Dona de baixa un jugador. Retirat com a jugador.
    PROCEDURE ges_retiraJugador(
        p_idJugador IN jugadors.idJugador%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_jugadorNF EXCEPTION;
        e_jugadorRetirat EXCEPTION;
        trobat NUMBER;
        m_estatActual jugadors.estat%TYPE;

    BEGIN

        -- Control ID jugador existent
        SELECT COUNT(*) INTO trobat FROM jugadors WHERE idJugador = p_idJugador;
        IF (trobat = 0) THEN
            RAISE e_jugadorNF;
        END IF;

        -- Control jugador retirat
        SELECT estat INTO m_estatActual FROM jugadors WHERE idJugador = p_idJugador;
        IF (m_estatActual = 'R') THEN
            RAISE e_jugadorRetirat;
        END IF;

        -- Reactivem el jugador
        UPDATE jugadors SET estat = 'R' WHERE idJugador = p_idJugador;

        -- Actualitza el subestat, per si un jugador que estaba de baixa es retira.
        UPDATE subestats SET dataFiReal = SYSDATE WHERE idJugador = p_idJugador AND dataFiReal IS NULL;

        -- Actualitza el nombre de jugadors de dwh_totalJugadors
        UPDATE dwh_totalJugadors SET total = (SELECT COUNT(*) FROM jugadors WHERE estat = 'A');

        DBMS_OUTPUT.PUT_LINE('Jugador [' || p_idJugador || '] marcat com a retirat');

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_retiraJugador', SYSDATE, p_idJugador, RSP);

    EXCEPTION
        WHEN e_jugadorNF THEN
            RSP := 'ERROR: ID de jugador [' || p_idJugador || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_retiraJugador', SYSDATE, p_idJugador, RSP);
        WHEN e_jugadorRetirat THEN
            RSP := 'ERROR: El jugador [' || p_idJugador || '] ja es troba retirat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_retiraJugador', SYSDATE, p_idJugador, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en baixa de jugador ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_retiraJugador', SYSDATE, p_idJugador, RSP);

    END ges_retiraJugador;

 -- Elimina un jugador de la BBDD
    PROCEDURE ges_esborraJugador(
        p_idJugador IN jugadors.idJugador%TYPE,
        RSP OUT logs.RSP%TYPE
        ) IS
        CURSOR c_idContracte IS
            SELECT idContracte
              FROM contractes
             WHERE idJugador = p_idJugador;

        e_jugadorNF EXCEPTION;
        trobat NUMBER;
        m_idContracte contractes.idContracte%TYPE;

    BEGIN
        -- Control ID jugador existent
        SELECT COUNT(*) INTO trobat FROM jugadors WHERE idJugador = p_idJugador;
        IF (trobat = 0) THEN
            RAISE e_jugadorNF;
        END IF;

        -- Esborrat de contractes
        -- Actualitzacio de les estadistiques de dwh_souMigJugadors
        OPEN c_idContracte;
        LOOP
            FETCH c_idContracte INTO m_idContracte;
            EXIT WHEN c_idContracte%NOTFOUND;

            pkg_gestioContractes.ges_esborraContracte(m_idContracte,RSP);

        END LOOP;

        CLOSE c_idContracte;

        -- Procedim a esborrar el jugador
        DELETE jugadors WHERE idJugador = p_idJugador;

        -- Actualitza el nombre de jugadors de dwh_totalJugadors
        UPDATE dwh_totalJugadors SET total = (SELECT COUNT(*) FROM jugadors WHERE estat = 'A');

        DBMS_OUTPUT.PUT_LINE('Jugador [' || p_idJugador || '] esborrat del sistema ');

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_esborraJugador', SYSDATE, p_idJugador, RSP);

    EXCEPTION
        WHEN e_jugadorNF THEN
            RSP := 'ERROR: ID de jugador [' || p_idJugador || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_esborraJugador', SYSDATE, p_idJugador, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en esborrat de jugador ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_esborraJugador', SYSDATE, p_idJugador, RSP);
    END ges_esborraJugador;

    -- Permet modificar les dades d'un jugador
    PROCEDURE ges_modificaJugador(
        p_idJugador IN jugadors.idJugador%TYPE,
        p_modalitat IN jugadors.modalitat%TYPE,
        p_alcada IN jugadors.alcada%TYPE,
        p_pes IN jugadors.pes%TYPE,
        p_posicio IN jugadors.posicio%TYPE,
        p_urlWeb IN jugadors.urlWeb%TYPE,
        p_urlVideo IN jugadors.urlVideo%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_jugadorNF EXCEPTION;
        e_modalitatCHK EXCEPTION;
        e_posicioCHK EXCEPTION;
        trobat NUMBER;

    BEGIN

        -- Control ID jugador existent
        SELECT COUNT(*) INTO trobat FROM jugadors WHERE idJugador = p_idJugador;
        IF (trobat = 0) THEN
            RAISE e_jugadorNF;
        END IF;

        -- Control modalitat
        IF (p_modalitat NOT IN ('N','C')) THEN
            RAISE e_modalitatCHK;
        END IF;

        -- Control posicio
        IF (p_posicio NOT IN (1,2,3,4,5)) THEN
            RAISE e_posicioCHK;
        END IF;

        -- Actualitzem les dades del jugador
        UPDATE jugadors
           SET modalitat = p_modalitat,
               alcada = p_alcada,
               pes = p_pes,
               posicio = p_posicio,
               urlWeb = p_urlWeb,
               urlVideo = p_urlVideo
         WHERE idJugador = p_idJugador;

        DBMS_OUTPUT.PUT_LINE('Jugador [' || p_idJugador || '] modificat');

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_modificaJugador', SYSDATE, p_idJugador, RSP);

    EXCEPTION
        WHEN e_jugadorNF THEN
            RSP := 'ERROR: ID de jugador [' || p_idJugador || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_modificaJugador', SYSDATE, p_idJugador, RSP);
        WHEN e_modalitatCHK THEN
            RSP := 'ERROR: Modalitat [' || p_modalitat || '] no vàlida';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_modificaJugador', SYSDATE, p_idJugador, RSP);
        WHEN e_posicioCHK THEN
            RSP := 'ERROR: Codi de posició [' || p_posicio || '] no vàlid';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_modificaJugador', SYSDATE, p_idJugador, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en alta de jugador ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_modificaJugador', SYSDATE, p_idJugador, RSP);

    END ges_modificaJugador;

    -- Dona d'alta o modifica les estaditiques per partit d'un jugador
    PROCEDURE ges_creaEstadistiques(
        p_idPartit IN partits.idPartit%TYPE,
        p_idJugador IN jugadors.idJugador%TYPE,
        p_mins IN convocats.mins%TYPE,
        p_t1I IN convocats.t1I%TYPE,
        p_t1E IN convocats.t1E%TYPE,
        p_t2I IN convocats.t2I%TYPE,
        p_t2E IN convocats.t2E%TYPE,
        p_t3I IN convocats.t3I%TYPE,
        p_t3E IN convocats.t3E%TYPE,
        p_rd IN convocats.rd%TYPE,
        p_ro IN convocats.ro%TYPE,
        p_asi IN convocats.asi%TYPE,
        p_tf IN convocats.tf%TYPE,
        p_tc IN convocats.tc%TYPE,
        p_pr IN convocats.pr%TYPE,
        p_pp IN convocats.pp%TYPE,
        p_fc IN convocats.fc%TYPE,
        p_fr IN convocats.fr%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_partitNF EXCEPTION;
        e_jugadorNF EXCEPTION;
        trobat NUMBER;
        m_idCompeticio competicions.idCompeticio%TYPE;
        m_anyCompeticio competicions.anyCompeticio%TYPE;
        m_nomCompeticio competicions.nom%TYPE;
        m_nom persones.nom%TYPE;
        m_cognoms persones.cognoms%TYPE;
        m_genere jugadors.genere%TYPE;
        m_modalitat jugadors.modalitat%TYPE;
        m_punts convocats.pt%TYPE;
        m_valoracioP convocats.valoracioP%TYPE;
    BEGIN
        -- Control ID partit existent
        SELECT COUNT(*) INTO trobat FROM partits WHERE idPartit = p_idPartit;
        IF (trobat = 0) THEN
            RAISE e_partitNF;
        END IF;

        -- Control ID jugador existent
        SELECT COUNT(*) INTO trobat FROM jugadors WHERE idJugador = p_idJugador;
        IF (trobat = 0) THEN
            RAISE e_jugadorNF;
        END IF;

        -- Primer mirar d'actualitzar les estadístiques de la convocatoria per si ja existeixen
        UPDATE convocats
           SET mins = p_mins,
               t1I = p_t1I,
               t1E = p_t1E,
               t2I = p_t2I,
               t2E = p_t2E,
               t3I = p_t3I,
               t3E = p_t3E,
               rd = p_rd,
               ro = p_ro,
               asi = p_asi,
               tf = p_tf,
               tc = p_tc,
               pr = p_pr,
               pp = p_pp,
               fc = p_fc,
               fr = p_fr
         WHERE idPartit = p_idPartit
           AND idJugador = p_idJugador;

        -- Si no existeix aquesta convocatoria la creem.
        IF (SQL%ROWCOUNT = 0) THEN
            -- Punts i valoracions calculats via trigger
            INSERT INTO convocats (idPartit,idJugador,mins,t1i,t1e,t2i,t2e,
                                   t3i,t3e,rd,ro,asi,tf,tc,pr,pp,fc,fr)
            VALUES (p_idPartit,p_idJugador,p_mins,p_t1i,p_t1e,p_t2i,p_t2e,
                    p_t3i,p_t3e,p_rd,p_ro,p_asi,p_tf,p_tc,p_pr,p_pp,p_fc,p_fr);
        END IF;

        -- Determinar dades del partit: competicio i any
        SELECT partits.idCompeticio, competicions.anyCompeticio, competicions.nom
          INTO m_idCompeticio, m_anyCompeticio, m_nomCompeticio
          FROM partits, competicions
         WHERE partits.idCompeticio = competicions.idCompeticio
           AND partits.idPartit = p_idPartit;

        -- Dades del jugador
        SELECT persones.nom, persones.cognoms, jugadors.genere, jugadors.modalitat
          INTO m_nom, m_cognoms, m_genere, m_modalitat
          FROM jugadors, persones
         WHERE jugadors.idPersona = persones.idPersona
           AND idJugador = p_idJugador;

        -- Actualitza les dades de dwh_anotadorsCompeticio.
        -- Donada una competició, el seu màxim anotador en la temporada en curs

        -- Calculem punts per jugador i competicio
        SELECT SUM(convocats.pt)
          INTO m_punts
          FROM convocats, partits
         WHERE convocats.idPartit = partits.idPartit
           AND convocats.idJugador = p_idJugador
           AND partits.idCompeticio = m_idCompeticio;

        -- Mirem d'actualitzar la estadistica per aquest jugador, any i competicio
        UPDATE dwh_anotadorsCompeticio
           SET punts = NVL(m_punts,0)
         WHERE idCompeticio = m_idCompeticio
           AND anyTemporada = m_anyCompeticio
           AND idJugador = p_idJugador;

        -- Si no existeix aquesta estadistica la creem.
        IF (SQL%ROWCOUNT = 0) THEN
            INSERT INTO dwh_anotadorsCompeticio (idCompeticio, nomCompeticio, anyTemporada, idJugador, nom, cognoms, punts)
            VALUES (m_idCompeticio, m_nomCompeticio, m_anyCompeticio, p_idJugador, m_nom, m_cognoms, NVL(m_punts,0));
        END IF;

        -- Actualitza les dades de dwh_millorJugador.
        -- Per a cada gènere i modalitat, el millor jugador del món en el darrer any.
        SELECT AVG(valoracioP)
          INTO m_valoracioP
          FROM convocats, partits, competicions
         WHERE convocats.idPartit = partits.idPartit
           AND partits.idCompeticio = competicions.idCompeticio
           AND convocats.idJugador = p_idJugador
           AND competicions.anyCompeticio = m_anyCompeticio;

        -- Mirem d'actualitzar les dades de dwh_millorJugador
        UPDATE dwh_millorJugador
           SET valoracioP = NVL(m_valoracioP,0)
         WHERE idJugador = p_idJugador
           AND genere = m_genere
           AND modalitat = m_modalitat
           AND anyTemporada = m_anyCompeticio;

        -- Si no existeix aquesta estadistica la creem.
        IF (SQL%ROWCOUNT = 0) THEN
            INSERT INTO dwh_millorJugador (genere, modalitat, idJugador, nom, cognoms, anyTemporada, valoracioP)
            VALUES (m_genere, m_modalitat, p_idJugador, m_nom, m_cognoms, m_anyCompeticio, NVL(m_valoracioP,0));
        END IF;

        DBMS_OUTPUT.PUT_LINE('Estadistiques actualitzades');

    EXCEPTION
        WHEN e_partitNF THEN
            RSP := 'ERROR: Partit [' || p_idPartit || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_creaEstadistiques', SYSDATE,
                    p_idPartit || ', ' || p_idJugador || ', ' || p_mins || ', ' || p_t1I || ', ' || p_t1E || ', ' ||
                    p_t2I || ', ' || p_t2E || ', ' || p_t3I || ', ' || p_t3E || ', ' || p_rd || ', ' || p_ro || ', ' ||
                    p_asi || ', ' || p_tf || ', ' || p_tc || ', ' || p_pr || ', ' || p_pp || ', ' || p_fc || ', ' || p_fr, RSP);
        WHEN e_jugadorNF THEN
            RSP := 'ERROR: ID de jugador [' || p_idJugador || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_creaEstadistiques', SYSDATE,
                    p_idPartit || ', ' || p_idJugador || ', ' || p_mins || ', ' || p_t1I || ', ' || p_t1E || ', ' ||
                    p_t2I || ', ' || p_t2E || ', ' || p_t3I || ', ' || p_t3E || ', ' || p_rd || ', ' || p_ro || ', ' ||
                    p_asi || ', ' || p_tf || ', ' || p_tc || ', ' || p_pr || ', ' || p_pp || ', ' || p_fc || ', ' || p_fr, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en modificar estaditiques ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_creaEstadistiques', SYSDATE,
                    p_idPartit || ', ' || p_idJugador || ', ' || p_mins || ', ' || p_t1I || ', ' || p_t1E || ', ' ||
                    p_t2I || ', ' || p_t2E || ', ' || p_t3I || ', ' || p_t3E || ', ' || p_rd || ', ' || p_ro || ', ' ||
                    p_asi || ', ' || p_tf || ', ' || p_tc || ', ' || p_pr || ', ' || p_pp || ', ' || p_fc || ', ' || p_fr, RSP);

    END ges_creaEstadistiques;

    -- Elimina una estadística d'un partit - jugador
    PROCEDURE ges_esborraEstadistica(
        p_idPartit IN partits.idPartit%TYPE,
        p_idJugador IN jugadors.idJugador%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_partitNF EXCEPTION;
        e_jugadorNF EXCEPTION;
        trobat NUMBER;
        m_idCompeticio competicions.idCompeticio%TYPE;
        m_anyCompeticio competicions.anyCompeticio%TYPE;
        m_punts convocats.pt%TYPE;
        m_genere jugadors.genere%TYPE;
        m_nom persones.nom%TYPE;
        m_cognoms persones.cognoms%TYPE;
        m_modalitat jugadors.modalitat%TYPE;
        m_valoracioP convocats.valoracioP%TYPE;
    BEGIN
        -- Control ID partit existent
        SELECT COUNT(*) INTO trobat FROM partits WHERE idPartit = p_idPartit;
        IF (trobat = 0) THEN
            RAISE e_partitNF;
        END IF;

        -- Control ID jugador existent
        SELECT COUNT(*) INTO trobat FROM jugadors WHERE idJugador = p_idJugador;
        IF (trobat = 0) THEN
            RAISE e_jugadorNF;
        END IF;

        -- Actualitzar les dades si existeix la convocatoria
        SELECT COUNT(*) INTO trobat FROM convocats WHERE idPartit = p_idPartit AND idJugador = p_idJugador;
        IF (trobat != 0) THEN

            -- Determinar dades del partit: competicio i any
            SELECT partits.idCompeticio, competicions.anyCompeticio
              INTO m_idCompeticio, m_anyCompeticio
              FROM partits, competicions
             WHERE partits.idCompeticio = competicions.idCompeticio
               AND partits.idPartit = p_idPartit;

            -- Dades del jugador
            SELECT persones.nom, persones.cognoms, jugadors.genere, jugadors.modalitat
              INTO m_nom, m_cognoms, m_genere, m_modalitat
              FROM jugadors, persones
             WHERE jugadors.idPersona = persones.idPersona
               AND idJugador = p_idJugador;

            -- Esborrar la convocatoria del sistema
            DELETE FROM convocats WHERE idPartit = p_idPartit AND idJugador = p_idJugador;

            -- Actualitza les dades de dwh_anotadorsCompeticio.
            -- Donada una competició, el seu màxim anotador en la temporada en curs

            -- Calculem punts per jugador i competicio
            SELECT SUM(convocats.pt)
              INTO m_punts
              FROM convocats, partits
             WHERE convocats.idPartit = partits.idPartit
               AND convocats.idJugador = p_idJugador
               AND partits.idCompeticio = m_idCompeticio;

            -- Mirem d'actualitzar la estadistica per aquest jugador, any i competicio
            UPDATE dwh_anotadorsCompeticio
               SET punts = NVL(m_punts,0)
             WHERE idCompeticio = m_idCompeticio
               AND anyTemporada = m_anyCompeticio
               AND idJugador = p_idJugador;

            -- Actualitza les dades de dwh_millorJugador.
            -- Per a cada gènere i modalitat, el millor jugador del món en el darrer any.
            SELECT AVG(valoracioP)
              INTO m_valoracioP
              FROM convocats, partits, competicions
             WHERE convocats.idPartit = partits.idPartit
               AND partits.idCompeticio = competicions.idCompeticio
               AND convocats.idJugador = p_idJugador
               AND competicions.anyCompeticio = m_anyCompeticio;

            -- Mirem d'actualitzar les dades de dwh_millorJugador
            UPDATE dwh_millorJugador
               SET valoracioP = NVL(m_valoracioP,0)
             WHERE idJugador = p_idJugador
               AND genere = m_genere
               AND modalitat = m_modalitat
               AND anyTemporada = m_anyCompeticio;

            -- Si no existeix aquesta estadistica la creem.
            IF (SQL%ROWCOUNT = 0) THEN
                INSERT INTO dwh_millorJugador (genere, modalitat, idJugador, nom, cognoms, anyTemporada, valoracioP)
                VALUES (m_genere, m_modalitat, p_idJugador, m_nom, m_cognoms, m_anyCompeticio, NVL(m_valoracioP,0));
            END IF;

        END IF;

        DBMS_OUTPUT.PUT_LINE('Estadistiques actualitzades');

    EXCEPTION
        WHEN e_partitNF THEN
            RSP := 'ERROR: Partit [' || p_idPartit || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_esborraEstadistica', SYSDATE, p_idPartit || ', ' || p_idJugador, RSP);
        WHEN e_jugadorNF THEN
            RSP := 'ERROR: ID de jugador [' || p_idJugador || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_esborraEstadistica', SYSDATE, p_idPartit || ', ' || p_idJugador, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en esborrar estadistiques ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_esborraEstadistica', SYSDATE, p_idPartit || ', ' || p_idJugador, RSP);

    END ges_esborraEstadistica;

    -- ---------------------------------------------------------------------
    -- NOM: ges_generaEstadistiques
    -- DESCRIPCIO: Genera estadistiques fictices per a un jugardor - partit
    -- PARAMETRES ENTRADA: p_idPartit - ID del partit
    --                     p_valoracio - ID del jugador
    -- PARAMETRES SORTIDA: RSP -  Resultat de l'execució del procediment
    --                           'OK' si s'ha executat correctament
    --                           'ERROR: ' + tipus_error en cas contrari
    --
    -- EXEMPLE: ges_generaEstadistiques(1,12,RSP)
    --          La sortida esperada dependra dels valors aleatoris calculats entre els marges establerts
    -- ---------------------------------------------------------------------
    PROCEDURE ges_generaEstadistiques(
        p_idPartit IN partits.idPartit%TYPE,
        p_idJugador IN jugadors.idJugador%TYPE,
        RSP OUT logs.RSP%TYPE
    )
    IS
        e_partitNF EXCEPTION;
        e_jugadorNF EXCEPTION;
        trobat NUMBER;

        m_minuts convocats.mins%TYPE;
        m_t1i convocats.t1i%TYPE;
        m_t1e convocats.t1e%TYPE;
        m_t2i convocats.t2i%TYPE;
        m_t2e convocats.t2e%TYPE;
        m_t3i convocats.t3i%TYPE;
        m_t3e convocats.t3e%TYPE;
        m_rd convocats.rd%TYPE;
        m_ro convocats.ro%TYPE;
        m_asi convocats.asi%TYPE;
        m_tf convocats.tf%TYPE;
        m_tc convocats.tc%TYPE;
        m_pr convocats.pr%TYPE;
        m_pp convocats.pp%TYPE;
        m_fc convocats.fc%TYPE;
        m_fr convocats.fr%TYPE;

    BEGIN
        -- Control per determinar si existeix el ID de partit
        SELECT COUNT(*) INTO trobat FROM partits WHERE idPartit = p_idPartit;
        IF (trobat = 0) THEN
            RAISE e_partitNF;
        END IF;

    -- Control per determinar si existeix el ID de partit
        SELECT COUNT(*) INTO trobat FROM jugadors WHERE idJugador = p_idJugador;
        IF (trobat = 0) THEN
            RAISE e_jugadorNF;
        END IF;

        -- Minuts jugats per partit entre 22 i 35
        m_minuts := TRUNC(DBMS_RANDOM.VALUE(22,35));

        -- tirs lliures entre 2 i 6 i 70% - 95%
        m_t1i:= TRUNC(DBMS_RANDOM.VALUE(2,6));
        m_t1e:= TRUNC(m_t1i * DBMS_RANDOM.VALUE(0.7,0.95));

        -- tirs de 2 entre 11 i 5 i 45% - 70%
        m_t2i:= TRUNC(DBMS_RANDOM.VALUE(2,11));
        m_t2e:= TRUNC(m_t1i * DBMS_RANDOM.VALUE(0.45,0.7));

        -- tirs de 3 entre 3 i 9 i 35% - 50%
        m_t3i:= TRUNC(DBMS_RANDOM.VALUE(3,9));
        m_t3e:= TRUNC(m_t1i * DBMS_RANDOM.VALUE(0.35,0.5));

        -- rd entre 3 i 7
        m_rd:= TRUNC(DBMS_RANDOM.VALUE(3,7));

        -- ro entre 1 i 3
        m_ro:= TRUNC(DBMS_RANDOM.VALUE(1,3));

        -- asi entre 3 i 7
        m_asi:= TRUNC(DBMS_RANDOM.VALUE(3,7));

        -- tf entre 0 i 2
        m_tf:= TRUNC(DBMS_RANDOM.VALUE(0,2));

        -- tc entre 0 i 2
        m_tc:= TRUNC(DBMS_RANDOM.VALUE(0,2));

        -- pr entre 1 i 3
        m_pr:= TRUNC(DBMS_RANDOM.VALUE(1,3));

        -- pp entre 1 i 3
        m_pp:= TRUNC(DBMS_RANDOM.VALUE(1,3));

        -- fr entre 2 i 6
        m_fr:= TRUNC(DBMS_RANDOM.VALUE(2,6));

        -- fc entre 2 i 6
        m_fc:= TRUNC(DBMS_RANDOM.VALUE(2,6));

        -- Creem les estadistiques a partir de la funcionalitat existent
        pkg_gestioJugadors.ges_creaEstadistiques(p_idPartit,p_idJugador,m_minuts,m_t1i,m_t1e,m_t2i,m_t2e,
                                                 m_t3i,m_t3e,m_rd,m_ro,m_asi,m_tf,m_tc,m_pr,m_pp,m_fc,m_fr,RSP);

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_generaEstadistiques', SYSDATE, p_idPartit || ',' || p_idJugador, RSP);

    EXCEPTION
        -- En cas de no existir el ID de partit
        WHEN e_partitNF THEN
            RSP := 'ERROR: Partit [' || p_idPartit || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_generaEstadistiques', SYSDATE, p_idPartit || ',' || p_idJugador, RSP);
        WHEN e_jugadorNF THEN
            RSP := 'ERROR: ID de jugador [' || p_idJugador || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_generaEstadistiques', SYSDATE, p_idPartit || ',' || p_idJugador, RSP);
        -- Altres errors
        WHEN OTHERS THEN
            ROLLBACK;
            RSP := 'ERROR: En la creacio de convocatoria ' || SQLERRM;
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_generaEstadistiques', SYSDATE, p_idPartit || ',' || p_idJugador, RSP);
    END ges_generaEstadistiques;

END pkg_gestioJugadors;