CREATE OR REPLACE PACKAGE pkg_gestioPartits
IS
    RSP logs.RSP%TYPE;

    -- Dona d'alta un partit en el sistema.
    PROCEDURE ges_altaPartit(
        p_idCompeticio IN competicions.idCompeticio%TYPE,
        p_idEquipLocal IN equips.idEquip%TYPE,
        p_idEquipVisitant IN equips.idEquip%TYPE,
        p_puntsLocal IN partits.puntsLocal%TYPE,
        p_puntsVisitant IN partits.puntsVisitant%TYPE,
        p_data IN partits.data%TYPE,
        p_hora IN partits.hora%TYPE,
        p_lloc IN partits.lloc%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Modifica les dades d'un partit.
    PROCEDURE ges_modificaPartit(
        p_idPartit IN partits.idPartit%TYPE,
        p_idCompeticio IN competicions.idCompeticio%TYPE,
        p_puntsLocal IN partits.puntsLocal%TYPE,
        p_puntsVisitant IN partits.puntsVisitant%TYPE,
        p_data IN partits.data%TYPE,
        p_hora IN partits.hora%TYPE,
        p_lloc IN partits.lloc%TYPE,
        RSP OUT logs.RSP%TYPE
    );

    -- Esborra un partit del sistema
    PROCEDURE ges_esborraPartit(
        p_idPartit IN partits.idPartit%TYPE,
        RSP OUT logs.RSP%TYPE
    );

END pkg_gestioPartits;
/
CREATE OR REPLACE PACKAGE BODY pkg_gestioPartits
IS

    -- Dona d'alta un partit en el sistema.
    PROCEDURE ges_altaPartit(
        p_idCompeticio IN competicions.idCompeticio%TYPE,
        p_idEquipLocal IN equips.idEquip%TYPE,
        p_idEquipVisitant IN equips.idEquip%TYPE,
        p_puntsLocal IN partits.puntsLocal%TYPE,
        p_puntsVisitant IN partits.puntsVisitant%TYPE,
        p_data IN partits.data%TYPE,
        p_hora IN partits.hora%TYPE,
        p_lloc IN partits.lloc%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_competicioNF EXCEPTION;
        e_equipNF EXCEPTION;
        e_null EXCEPTION;
        e_mateixEquip EXCEPTION;
        trobat NUMBER;
        m_nomLocal equips.nom%TYPE;
        m_nomVisitant equips.nom%TYPE;
        m_nomCompeticio competicions.nom%TYPE;
    BEGIN
        -- Control ID competicio existent
        SELECT COUNT(*) INTO trobat FROM competicions WHERE idCompeticio = p_idCompeticio;
        IF (trobat = 0) THEN
            RAISE e_competicioNF;
        END IF;

        -- Control equip Local existent
        SELECT COUNT(*) INTO trobat FROM equips WHERE idEquip = p_idEquipLocal;
        IF (trobat = 0) THEN
            RAISE e_equipNF;
        END IF;

        -- Control equip visitant existent
        SELECT COUNT(*) INTO trobat FROM equips WHERE idEquip = p_idEquipVisitant;
        IF (trobat = 0) THEN
            RAISE e_equipNF;
        END IF;

        -- Control dades obligatories
        IF (p_data IS NULL OR p_hora IS NULL OR p_lloc IS NULL) THEN
            RAISE e_null;
        END IF;

        -- Controlar que el equip visitant i local no siguin el mateix
        IF (p_idEquipLocal = p_idEquipVisitant) THEN
            RAISE e_mateixEquip;
        END IF;

        -- Si no hi ha cap error, podem crear el partit
        INSERT INTO partits (idCompeticio, equipLocal, equipVisitant,
                             puntsLocal, puntsVisitant, data, hora, lloc)
        VALUES (p_idCompeticio, p_idEquipLocal, p_idEquipVisitant,
                NVL(p_puntsLocal,0), NVL(p_puntsVisitant,0), p_data, p_hora, p_lloc);

        -- Actualitza les dades de dwh_millorsEquips.
        -- Els millors equips ofensius i defensius
        -- Primer actualitzar per si ja existeix registre per equip, competicio i temporada.

        -- Dades de la competicio
        SELECT nom INTO m_nomCompeticio FROM competicions WHERE idCompeticio = p_idCompeticio;
        -- Dades dels equips
        SELECT nom INTO m_nomLocal FROM equips WHERE idEquip = p_idEquipLocal;
        SELECT nom INTO m_nomVisitant FROM equips WHERE idEquip = p_idEquipVisitant;

        -- Equip Local, punts ofensius
        UPDATE dwh_millorsEquips
           SET punts = punts + p_puntsLocal
         WHERE idEquip = p_idEquipLocal
           AND idCompeticio = p_idCompeticio
           AND anyTemporada = EXTRACT(YEAR FROM p_data)
           AND tipus = 'O';

        -- Si no existeix aquesta estadistica al DWH la donem d'alta.
        IF (SQL%ROWCOUNT = 0) THEN
            INSERT INTO dwh_millorsEquips (idCompeticio, nomCompeticio, anyTemporada, idEquip, nomEquip, tipus, punts)
            VALUES (p_idCompeticio, m_nomCompeticio, EXTRACT(YEAR FROM p_data), p_idEquipLocal, m_nomLocal, 'O', p_puntsLocal);
        END IF;

        -- Equip Local, punts defensius
        UPDATE dwh_millorsEquips
           SET punts = punts + p_puntsVisitant
         WHERE idEquip = p_idEquipLocal
           AND idCompeticio = p_idCompeticio
           AND anyTemporada = EXTRACT(YEAR FROM p_data)
           AND tipus = 'D';

        -- Si no existeix aquesta estadistica al DWH la donem d'alta.
        IF (SQL%ROWCOUNT = 0) THEN
            INSERT INTO dwh_millorsEquips (idCompeticio, nomCompeticio, anyTemporada, idEquip, nomEquip, tipus, punts)
            VALUES (p_idCompeticio, m_nomCompeticio, EXTRACT(YEAR FROM p_data), p_idEquipLocal, m_nomLocal, 'D', p_puntsVisitant);
        END IF;

        -- Equip Visitant, punts ofensius
        UPDATE dwh_millorsEquips
           SET punts = punts + p_puntsVisitant
         WHERE idEquip = p_idEquipVisitant
           AND idCompeticio = p_idCompeticio
           AND anyTemporada = EXTRACT(YEAR FROM p_data)
           AND tipus = 'O';

        -- Si no existeix aquesta estadistica al DWH la donem d'alta.
        IF (SQL%ROWCOUNT = 0) THEN
            INSERT INTO dwh_millorsEquips (idCompeticio, nomCompeticio, anyTemporada, idEquip, nomEquip, tipus, punts)
            VALUES (p_idCompeticio, m_nomCompeticio, EXTRACT(YEAR FROM p_data), p_idEquipVisitant, m_nomVisitant, 'O', p_puntsVisitant);
        END IF;

        -- Equip Visitant, punts defensius
        UPDATE dwh_millorsEquips
           SET punts = punts + p_puntsLocal
         WHERE idEquip = p_idEquipVisitant
           AND idCompeticio = p_idCompeticio
           AND anyTemporada = EXTRACT(YEAR FROM p_data)
           AND tipus = 'D';

        -- Si no existeix aquesta estadistica al DWH la donem d'alta.
        IF (SQL%ROWCOUNT = 0) THEN
            INSERT INTO dwh_millorsEquips (idCompeticio, nomCompeticio, anyTemporada, idEquip, nomEquip, tipus, punts)
            VALUES (p_idCompeticio, m_nomCompeticio, EXTRACT(YEAR FROM p_data), p_idEquipVisitant, m_nomVisitant, 'D', p_puntsLocal);
        END IF;

        DBMS_OUTPUT.PUT_LINE('Partit creat');

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_altaPartit', SYSDATE,
                p_idCompeticio || ', ' || p_idEquipLocal || ', ' || p_idEquipVisitant || ', ' ||
                p_puntsLocal || ', ' || p_puntsVisitant || ', ' || p_data || ', ' || p_hora || ', ' || p_lloc, RSP);

    EXCEPTION
        WHEN e_competicioNF THEN
            RSP := 'ERROR: Competicio [' || p_idCompeticio || '] no trobada';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaPartit', SYSDATE,
                p_idCompeticio || ', ' || p_idEquipLocal || ', ' || p_idEquipVisitant || ', ' ||
                p_puntsLocal || ', ' || p_puntsVisitant || ', ' || p_data || ', ' || p_hora || ', ' || p_lloc, RSP);
        WHEN e_equipNF THEN
            RSP := 'ERROR: Equip no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaPartit', SYSDATE,
                p_idCompeticio || ', ' || p_idEquipLocal || ', ' || p_idEquipVisitant || ', ' ||
                p_puntsLocal || ', ' || p_puntsVisitant || ', ' || p_data || ', ' || p_hora || ', ' || p_lloc, RSP);
        WHEN e_null THEN
            RSP := 'ERROR: Data, hora i lloc son obligatoris';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaPartit', SYSDATE,
                p_idCompeticio || ', ' || p_idEquipLocal || ', ' || p_idEquipVisitant || ', ' ||
                p_puntsLocal || ', ' || p_puntsVisitant || ', ' || p_data || ', ' || p_hora || ', ' || p_lloc, RSP);
        WHEN e_mateixEquip THEN
            RSP := 'ERROR: Equip visitant i local han de ser diferents';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaPartit', SYSDATE,
                p_idCompeticio || ', ' || p_idEquipLocal || ', ' || p_idEquipVisitant || ', ' ||
                p_puntsLocal || ', ' || p_puntsVisitant || ', ' || p_data || ', ' || p_hora || ', ' || p_lloc, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en alta de partit ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_altaPartit', SYSDATE,
                p_idCompeticio || ', ' || p_idEquipLocal || ', ' || p_idEquipVisitant || ', ' ||
                p_puntsLocal || ', ' || p_puntsVisitant || ', ' || p_data || ', ' || p_hora || ', ' || p_lloc, RSP);

    END ges_altaPartit;

    -- Modifica les dades d'un partit.
    PROCEDURE ges_modificaPartit(
        p_idPartit IN partits.idPartit%TYPE,
        p_idCompeticio IN competicions.idCompeticio%TYPE,
        p_puntsLocal IN partits.puntsLocal%TYPE,
        p_puntsVisitant IN partits.puntsVisitant%TYPE,
        p_data IN partits.data%TYPE,
        p_hora IN partits.hora%TYPE,
        p_lloc IN partits.lloc%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_partitNF EXCEPTION;
        e_competicioNF EXCEPTION;
        e_null EXCEPTION;
        trobat NUMBER;
        m_nomLocal equips.nom%TYPE;
        m_nomVisitant equips.nom%TYPE;
        m_nomCompeticio competicions.nom%TYPE;
        m_puntsOffEq1L INTEGER;
        m_puntsOffEq1V INTEGER;
        m_puntsDefEq1L INTEGER;
        m_puntsDefEq1V INTEGER;
        m_puntsOffEq2L INTEGER;
        m_puntsOffEq2v INTEGER;
        m_puntsDefEq2L INTEGER;
        m_puntsDefEq2v INTEGER;
        m_idEquipLocal equips.idEquip%TYPE;
        m_idEquipVisitant equips.idEquip%TYPE;

    BEGIN

        -- Control ID partit existent
        SELECT COUNT(*) INTO trobat FROM partits WHERE idPartit = p_idPartit;
        IF (trobat = 0) THEN
            RAISE e_partitNF;
        END IF;

        -- Control ID competicio existent
        SELECT COUNT(*) INTO trobat FROM competicions WHERE idCompeticio = p_idCompeticio;
        IF (trobat = 0) THEN
            RAISE e_competicioNF;
        END IF;

        -- Control dades obligatories
        IF (p_data IS NULL OR p_hora IS NULL OR p_lloc IS NULL) THEN
            RAISE e_null;
        END IF;

        -- Si no hi ha cap error, podem actualitzar les dades del partit
        UPDATE partits
           SET idCompeticio = p_idCompeticio,
               puntsLocal = p_puntsLocal,
               puntsVisitant = p_puntsVisitant,
               data = p_data,
               hora = p_hora,
               lloc = p_lloc
         WHERE idPartit = p_idPartit;

        -- Recuperar els equips del partit
        SELECT equipLocal, equipVisitant INTO m_idEquipLocal, m_idEquipVisitant FROM partits WHERE idPartit = p_idPartit;

        -- Actualitza les dades de dwh_millorsEquips.
        -- Els millors equips ofensius i defensius
        -- Primer actualitzar per si ja existeix registre per equip, competicio i temporada.

        -- Dades de la competicio
        SELECT nom INTO m_nomCompeticio FROM competicions WHERE idCompeticio = p_idCompeticio;
        -- Dades dels equips
        SELECT nom INTO m_nomLocal FROM equips WHERE idEquip = m_idEquipLocal;
        SELECT nom INTO m_nomVisitant FROM equips WHERE idEquip = m_idEquipVisitant;

        -- Equip 1
        -- Punts a favor com local
        SELECT SUM(puntsLocal)
          INTO m_puntsOffEq1L
          FROM partits
         WHERE equipLocal = m_idEquipLocal
           AND idCompeticio = p_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM p_data);

        -- Punts a favor com a visitant
        SELECT SUM(puntsVisitant)
          INTO m_puntsOffEq1V
          FROM partits
         WHERE equipVisitant = m_idEquipLocal
           AND idCompeticio = p_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM p_data);

        -- Actualitzar punts ofensius de l'equip local
        UPDATE dwh_millorsEquips
           SET punts = m_puntsOffEq1L + m_puntsOffEq1V
         WHERE idEquip = m_idEquipLocal
           AND idCompeticio = p_idCompeticio
           AND anyTemporada = EXTRACT(YEAR FROM p_data)
           AND tipus = 'O';

        -- Punts en contra com local
        SELECT SUM(puntsVisitant)
          INTO m_puntsDefEq1L
          FROM partits
         WHERE equipLocal = m_idEquipLocal
           AND idCompeticio = p_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM p_data);

        -- Punts en contra com a visitant
        SELECT SUM(puntsLocal)
          INTO m_puntsDefEq1V
          FROM partits
         WHERE equipVisitant = m_idEquipLocal
           AND idCompeticio = p_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM p_data);

        -- Actualitzar punts defensius de l'equip local
        UPDATE dwh_millorsEquips
           SET punts = m_puntsDefEq1L + m_puntsDefEq1V
         WHERE idEquip = m_idEquipLocal
           AND idCompeticio = p_idCompeticio
           AND anyTemporada = EXTRACT(YEAR FROM p_data)
           AND tipus = 'D';

        -- Equip 2
        -- Punts a favor com local
        SELECT SUM(puntsLocal)
          INTO m_puntsOffEq2L
          FROM partits
         WHERE equipLocal = m_idEquipVisitant
           AND idCompeticio = p_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM p_data);

        -- Punts a favor com a visitant
        SELECT SUM(puntsVisitant)
          INTO m_puntsOffEq2V
          FROM partits
         WHERE equipVisitant = m_idEquipVisitant
           AND idCompeticio = p_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM p_data);

        -- Actualitzar punts ofensius de l'equip local
        UPDATE dwh_millorsEquips
           SET punts = m_puntsOffEq2L + m_puntsOffEq2V
         WHERE idEquip = m_idEquipVisitant
           AND idCompeticio = p_idCompeticio
           AND anyTemporada = EXTRACT(YEAR FROM p_data)
           AND tipus = 'O';

        -- Punts en contra com local
        SELECT SUM(puntsVisitant)
          INTO m_puntsDefEq2L
          FROM partits
         WHERE equipLocal = m_idEquipVisitant
           AND idCompeticio = p_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM p_data);

        -- Punts en contra com a visitant
        SELECT SUM(puntsLocal)
          INTO m_puntsDefEq2V
          FROM partits
         WHERE equipVisitant = m_idEquipVisitant
           AND idCompeticio = p_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM p_data);

        -- Actualitzar punts defensius de l'equip local
        UPDATE dwh_millorsEquips
           SET punts = m_puntsDefEq2L + m_puntsDefEq2V
         WHERE idEquip = m_idEquipVisitant
           AND idCompeticio = p_idCompeticio
           AND anyTemporada = EXTRACT(YEAR FROM p_data)
           AND tipus = 'D';

        DBMS_OUTPUT.PUT_LINE('Partit modificat');

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_modificaPartit', SYSDATE,
                p_idPartit || ', ' || p_idCompeticio || ', ' || 
                p_puntsLocal || ', ' || p_puntsVisitant || ', ' || p_data || ', ' || p_hora || ', ' || p_lloc, RSP);

    EXCEPTION
        WHEN e_partitNF THEN
            RSP := 'ERROR: Partit [' || p_idPartit || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_modificaPartit', SYSDATE,
                p_idPartit || ', ' || p_idCompeticio || ', ' || 
                p_puntsLocal || ', ' || p_puntsVisitant || ', ' || p_data || ', ' || p_hora || ', ' || p_lloc, RSP);
        WHEN e_competicioNF THEN
            RSP := 'ERROR: Competicio [' || p_idCompeticio || '] no trobada';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_modificaPartit', SYSDATE,
                p_idPartit || ', ' || p_idCompeticio || ', ' || 
                p_puntsLocal || ', ' || p_puntsVisitant || ', ' || p_data || ', ' || p_hora || ', ' || p_lloc, RSP);
        WHEN e_null THEN
            RSP := 'ERROR: Data, hora i lloc son obligatoris';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_modificaPartit', SYSDATE,
                p_idPartit || ', ' || p_idCompeticio || ', ' || 
                p_puntsLocal || ', ' || p_puntsVisitant || ', ' || p_data || ', ' || p_hora || ', ' || p_lloc, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en modificacio de partit ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_modificaPartit', SYSDATE,
                p_idPartit || ', ' || p_idCompeticio || ', ' || 
                p_puntsLocal || ', ' || p_puntsVisitant || ', ' || p_data || ', ' || p_hora || ', ' || p_lloc, RSP);

    END ges_modificaPartit;

    -- Esborra un partit del sistema
    PROCEDURE ges_esborraPartit(
        p_idPartit IN partits.idPartit%TYPE,
        RSP OUT logs.RSP%TYPE)
    IS
        e_partitNF EXCEPTION;
        trobat NUMBER;
        m_idCompeticio partits.idCompeticio%TYPE;
        m_equipLocal partits.equipLocal%TYPE;
        m_equipVisitant partits.equipVisitant%TYPE;
        m_data partits.data%TYPE;
        m_nomLocal equips.nom%TYPE;
        m_nomVisitant equips.nom%TYPE;
        m_nomCompeticio competicions.nom%TYPE;
        m_puntsOffEq1L INTEGER;
        m_puntsOffEq1V INTEGER;
        m_puntsDefEq1L INTEGER;
        m_puntsDefEq1V INTEGER;
        m_puntsOffEq2L INTEGER;
        m_puntsOffEq2v INTEGER;
        m_puntsDefEq2L INTEGER;
        m_puntsDefEq2v INTEGER;
        m_idJugador jugadors.idJugador%TYPE;
        CURSOR c_idJugador IS
            SELECT idJugador
              FROM convocats
             WHERE idPartit = p_idPartit;
    BEGIN
        -- Control ID partit existent
        SELECT COUNT(*) INTO trobat FROM partits WHERE idPartit = p_idPartit;
        IF (trobat = 0) THEN
            RAISE e_partitNF;
        END IF;

        -- Esborrat de convocatories
        -- Actualitzacio de les estadistiques del DWH que depenen d'un jugador
        OPEN c_idJugador;
        LOOP
            FETCH c_idJugador INTO m_idJugador;
            EXIT WHEN c_idJugador%NOTFOUND;

            pkg_gestioJugadors.ges_esborraEstadistica(p_idPartit, m_idJugador,RSP);

        END LOOP;

        CLOSE c_idJugador;

        -- recuperar les dades del partit per poder actualitzar les estadístiques
        SELECT idCompeticio, equipLocal, equipVisitant, data
          INTO m_idCompeticio, m_equipLocal, m_equipVisitant, m_data
          FROM partits
         WHERE idPartit = p_idPartit;

        -- Esborrem el partit
        DELETE FROM partits WHERE idPartit = p_idPartit;

        -- Actualitza les dades de dwh_millorsEquips.
        -- Els millors equips ofensius i defensius
        -- Primer actualitzar per si ja existeix registre per equip, competicio i temporada.

        -- Dades de la competicio
        SELECT nom INTO m_nomCompeticio FROM competicions WHERE idCompeticio = m_idCompeticio;
        -- Dades dels equips
        SELECT nom INTO m_nomLocal FROM equips WHERE idEquip = m_equipLocal;
        SELECT nom INTO m_nomVisitant FROM equips WHERE idEquip = m_equipVisitant;

        -- Equip 1
        -- Punts a favor com local
        SELECT SUM(puntsLocal)
          INTO m_puntsOffEq1L
          FROM partits
         WHERE equipLocal = m_equipLocal
           AND idCompeticio = m_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM m_data);

        -- Punts a favor com a visitant
        SELECT SUM(puntsVisitant)
          INTO m_puntsOffEq1V
          FROM partits
         WHERE equipVisitant = m_equipLocal
           AND idCompeticio = m_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM m_data);

        -- Actualitzar punts ofensius de l'equip local
        UPDATE dwh_millorsEquips
           SET punts = NVL(m_puntsOffEq1L,0) + NVL(m_puntsOffEq1V,0)
         WHERE idEquip = m_equipLocal
           AND idCompeticio = m_idCompeticio
           AND anyTemporada = EXTRACT(YEAR FROM m_data)
           AND tipus = 'O';

        -- Punts en contra com local
        SELECT SUM(puntsVisitant)
          INTO m_puntsDefEq1L
          FROM partits
         WHERE equipLocal = m_equipLocal
           AND idCompeticio = m_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM m_data);

        -- Punts en contra com a visitant
        SELECT SUM(puntsLocal)
          INTO m_puntsDefEq1V
          FROM partits
         WHERE equipVisitant = m_equipLocal
           AND idCompeticio = m_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM m_data);

        -- Actualitzar punts defensius de l'equip local
        UPDATE dwh_millorsEquips
           SET punts = NVL(m_puntsDefEq1L,0) + NVL(m_puntsDefEq1V,0)
         WHERE idEquip = m_equipLocal
           AND idCompeticio = m_idCompeticio
           AND anyTemporada = EXTRACT(YEAR FROM m_data)
           AND tipus = 'D';

        -- Equip 2
        -- Punts a favor com local
        SELECT SUM(puntsLocal)
          INTO m_puntsOffEq2L
          FROM partits
         WHERE equipLocal = m_EquipVisitant
           AND idCompeticio = m_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM m_data);

        -- Punts a favor com a visitant
        SELECT SUM(puntsVisitant)
          INTO m_puntsOffEq2V
          FROM partits
         WHERE equipVisitant = m_EquipVisitant
           AND idCompeticio = m_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM m_data);

        -- Actualitzar punts ofensius de l'equip local
        UPDATE dwh_millorsEquips
           SET punts = NVL(m_puntsOffEq2L,0) + NVL(m_puntsOffEq2V,0)
         WHERE idEquip = m_EquipVisitant
           AND idCompeticio = m_idCompeticio
           AND anyTemporada = EXTRACT(YEAR FROM m_data)
           AND tipus = 'O';

        -- Punts en contra com local
        SELECT SUM(puntsVisitant)
          INTO m_puntsDefEq2L
          FROM partits
         WHERE equipLocal = m_EquipVisitant
           AND idCompeticio = m_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM m_data);

        -- Punts en contra com a visitant
        SELECT SUM(puntsLocal)
          INTO m_puntsDefEq2V
          FROM partits
         WHERE equipVisitant = m_EquipVisitant
           AND idCompeticio = m_idCompeticio
           AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM m_data);

        -- Actualitzar punts defensius de l'equip local
        UPDATE dwh_millorsEquips
           SET punts = NVL(m_puntsDefEq2L,0) + NVL(m_puntsDefEq2V,0)
         WHERE idEquip = m_EquipVisitant
           AND idCompeticio = m_idCompeticio
           AND anyTemporada = EXTRACT(YEAR FROM m_data)
           AND tipus = 'D';

        DBMS_OUTPUT.PUT_LINE('Partit esborrat');

        -- Log d'execucions
        RSP := 'OK';
        INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
        VALUES ('ges_esborraPartit', SYSDATE, p_idPartit, RSP);

    EXCEPTION
        WHEN e_partitNF THEN
            RSP := 'ERROR: Partit [' || p_idPartit || '] no trobat';
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_esborraPartit', SYSDATE, p_idPartit, RSP);
        WHEN OTHERS THEN
            RSP := 'ERROR: Error en esborrat de partit ' || SQLERRM;
            DBMS_OUTPUT.PUT_LINE(RSP);
            INSERT INTO logs (nomProc, data, paramsEntrada, RSP)
            VALUES ('ges_esborraPartit', SYSDATE, p_idPartit, RSP);

    END ges_esborraPartit;

END pkg_gestioPartits;