-- =====================================================================
-- Creaci de l'usuari agb
-- =====================================================================
--CREATE USER AGB IDENTIFIED BY agb
--DEFAULT TABLESPACE USERS
--QUOTA UNLIMITED ON USERS
--TEMPORARY TABLESPACE TEMP;

--GRANT CREATE SESSION, CREATE DATABASE LINK, CREATE TABLE, CREATE SEQUENCE,
--      CREATE PROCEDURE,  CREATE TRIGGER, CREATE VIEW
--TO AGB;
--COMMIT;

-- =====================================================================
--  CREACIO DE TAULES, CONSTRAINTS, SEQENCIES i TRIGGERS
-- =====================================================================
-- ---------------------------------------------------------------------
-- PAISOS
-- ---------------------------------------------------------------------
CREATE TABLE paisos(
    idPais CHAR(3) NOT NULL,
    nom VARCHAR2(100) NOT NULL
);
CREATE UNIQUE INDEX p_paisos ON paisos(idPais);
ALTER TABLE paisos ADD CONSTRAINT p_paisos PRIMARY KEY (idPais);

-- ---------------------------------------------------------------------
-- MUNICIPIS
-- ---------------------------------------------------------------------
CREATE TABLE municipis(
    idMunicipi INTEGER NOT NULL,
    idPais CHAR(3) NOT NULL,
    nom VARCHAR2(100) NOT NULL
);
CREATE UNIQUE INDEX p_municipis ON municipis(idMunicipi);
ALTER TABLE municipis ADD CONSTRAINT p_municipis PRIMARY KEY (idMunicipi);

CREATE INDEX f_municipis1 ON municipis(idPais);
ALTER TABLE municipis ADD CONSTRAINT f_municipis1 FOREIGN KEY (idPais) REFERENCES paisos(idPais);

CREATE SEQUENCE s_municipis1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER municipis_ins BEFORE INSERT ON municipis
    FOR EACH ROW
    BEGIN
        SELECT s_municipis1.NEXTVAL INTO :NEW.idMunicipi FROM DUAL;
    END;
/
-- ---------------------------------------------------------------------
-- FEDERACIONS
-- ---------------------------------------------------------------------
CREATE TABLE federacions(
    idFederacio INTEGER NOT NULL,
    idPais CHAR(3) NOT NULL,
    nom VARCHAR2(100) NOT NULL
);
CREATE UNIQUE INDEX p_federacions ON federacions(idFederacio);
ALTER TABLE federacions ADD CONSTRAINT p_federacions PRIMARY KEY (idFederacio);

CREATE INDEX f_federacions1 ON federacions(idPais);
ALTER TABLE federacions ADD CONSTRAINT f_federacions1 FOREIGN KEY (idPais) REFERENCES paisos(idPais);

CREATE SEQUENCE s_federacions1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER federacions_ins BEFORE INSERT ON federacions
    FOR EACH ROW
    BEGIN
        SELECT s_federacions1.NEXTVAL INTO :NEW.idFederacio FROM DUAL;
    END;
/
-- ---------------------------------------------------------------------
-- PERSONES
-- ---------------------------------------------------------------------
CREATE TABLE persones(
    idPersona INTEGER NOT NULL,
    idPais CHAR(3) NOT NULL,
    nom VARCHAR2(50) NOT NULL,
    cognoms VARCHAR2(50) NOT NULL,
    dataNaixement DATE NOT NULL,
    estat CHAR(1) DEFAULT 'A' NOT NULL
);
CREATE UNIQUE INDEX p_persones ON persones(idPersona);
ALTER TABLE persones ADD CONSTRAINT p_persones PRIMARY KEY (idPersona);

CREATE INDEX f_persones1 ON persones(idPais);
ALTER TABLE persones ADD CONSTRAINT f_persones1 FOREIGN KEY (idPais) REFERENCES paisos(idPais);

ALTER TABLE persones ADD CONSTRAINT c_persones1 CHECK (estat IN ('A', 'B'));

CREATE SEQUENCE s_persones1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER persones_ins BEFORE INSERT ON persones
    FOR EACH ROW
    BEGIN
        SELECT s_persones1.NEXTVAL INTO :NEW.idPersona FROM DUAL;
    END;
/
-- ---------------------------------------------------------------------
-- JUGADORS
-- ---------------------------------------------------------------------
CREATE TABLE jugadors(
    idJugador INTEGER NOT NULL,
    idPersona INTEGER NOT NULL,
    idFederacio INTEGER NOT NULL,
    numFederat NUMBER(10) NOT NULL,
    genere char(1) NOT NULL,
    modalitat CHAR(1) NOT NULL,
    alcada NUMBER(5),
    pes NUMBER(6,2),
    posicio NUMBER(5) NOT NULL,
    urlWeb VARCHAR2(250),
    urlVideo VARCHAR2(250),
    estat CHAR(1) DEFAULT 'A' NOT NULL
);
CREATE UNIQUE INDEX p_jugadors ON jugadors(idJugador);
ALTER TABLE jugadors ADD CONSTRAINT p_jugadors PRIMARY KEY (idJugador);

CREATE INDEX f_jugadors1 ON jugadors(idPersona);
ALTER TABLE jugadors ADD CONSTRAINT f_jugadors1 FOREIGN KEY (idPersona) REFERENCES persones(idPersona);

CREATE INDEX f_jugadors2 ON jugadors(idFederacio);
ALTER TABLE jugadors ADD CONSTRAINT f_jugadors2 FOREIGN KEY (idFederacio) REFERENCES federacions(idFederacio);

ALTER TABLE jugadors ADD CONSTRAINT c_jugadors1 CHECK (posicio >= 1 AND posicio <= 5);
ALTER TABLE jugadors ADD CONSTRAINT c_jugadors2 CHECK (estat IN ('A', 'B', 'R'));
ALTER TABLE jugadors ADD CONSTRAINT c_jugadors3 CHECK (genere IN ('M','F'));
ALTER TABLE jugadors ADD CONSTRAINT c_jugadors4 CHECK (modalitat IN ('N','C'));

CREATE SEQUENCE s_jugadors1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER jugadors_ins BEFORE INSERT ON jugadors
    FOR EACH ROW
    BEGIN
        SELECT s_jugadors1.NEXTVAL INTO :NEW.idJugador FROM DUAL;
    END;
/
-- ---------------------------------------------------------------------
-- REPRESENTANTSJUGADOR
-- ---------------------------------------------------------------------
CREATE TABLE representantsJugador(
    idRepresentant INTEGER NOT NULL,
    idPersona INTEGER NOT NULL,
    idJugador INTEGER NOT NULL,
    dataInici DATE NOT NULL,
    dataFi DATE,
    estat CHAR(1) DEFAULT 'A' NOT NULL
);
CREATE UNIQUE INDEX p_representantsJugador ON representantsJugador(idRepresentant);
ALTER TABLE representantsJugador ADD CONSTRAINT p_jrepresentantsJugador PRIMARY KEY (idRepresentant);

CREATE INDEX f_representantsJugador1 ON representantsJugador(idPersona);
ALTER TABLE representantsJugador ADD CONSTRAINT f_representantsJugador1 FOREIGN KEY (idPersona) REFERENCES persones(idPersona);

CREATE INDEX f_representantsJugador2 ON representantsJugador(idJugador);
ALTER TABLE representantsJugador ADD CONSTRAINT f_representantsJugador2 FOREIGN KEY (idJugador) REFERENCES jugadors(idJugador) ON DELETE CASCADE;

ALTER TABLE representantsJugador ADD CONSTRAINT c_representantsJugador1 CHECK (dataInici <= dataFi);
ALTER TABLE representantsJugador ADD CONSTRAINT c_representantsJugador2 CHECK (estat IN ('A', 'B'));

CREATE SEQUENCE s_representantsJugador1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER representantsJugador_ins BEFORE INSERT ON representantsJugador
    FOR EACH ROW
    BEGIN
        SELECT s_representantsJugador1.NEXTVAL INTO :NEW.idRepresentant FROM DUAL;
    END;
/
CREATE OR REPLACE TRIGGER representantsJugador_upd BEFORE UPDATE ON representantsJugador
    FOR EACH ROW
    BEGIN
        IF (:NEW.estat != :OLD.estat AND :NEW.estat = 'B') THEN
            :NEW.dataFI := SYSDATE;
        END IF;
        IF (:NEW.estat != :OLD.estat AND :NEW.estat = 'A') THEN
            :NEW.dataFI := NULL;
        END IF;
    END;
/
-- ---------------------------------------------------------------------
-- REPRESENTANTSLEGALS
-- ---------------------------------------------------------------------
CREATE TABLE representantsLegals(
    idRepresentant INTEGER NOT NULL,
    idPersona INTEGER NOT NULL,
    dataInici DATE NOT NULL,
    dataFi DATE,
    estat CHAR(1) DEFAULT 'A' NOT NULL
);
CREATE UNIQUE INDEX p_representantsLegals ON representantsLegals(idRepresentant);
ALTER TABLE representantsLegals ADD CONSTRAINT p_jrepresentantsLegals PRIMARY KEY (idRepresentant);

CREATE INDEX f_representantsLegals1 ON representantsLegals(idPersona);
ALTER TABLE representantsLegals ADD CONSTRAINT f_representantsLegals1 FOREIGN KEY (idPersona) REFERENCES persones(idPersona);

ALTER TABLE representantsLegals ADD CONSTRAINT c_representantsLegals1 CHECK (dataInici <= dataFi);
ALTER TABLE representantsLegals ADD CONSTRAINT c_representantsLegals2 CHECK (estat IN ('A', 'B'));

CREATE SEQUENCE s_representantsLegals1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER representantsLegals_ins BEFORE INSERT ON representantsLegals
    FOR EACH ROW
    BEGIN
        SELECT s_representantsLegals1.NEXTVAL INTO :NEW.idRepresentant FROM DUAL;
    END;
/
CREATE OR REPLACE TRIGGER representantsLegals_upd BEFORE UPDATE ON representantsLegals
    FOR EACH ROW
    BEGIN
        IF (:NEW.estat != :OLD.estat AND :NEW.estat = 'B') THEN
            :NEW.dataFI := SYSDATE;
        END IF;
        IF (:NEW.estat != :OLD.estat AND :NEW.estat = 'A') THEN
            :NEW.dataFI := NULL;
        END IF;
    END;
/
-- ---------------------------------------------------------------------
-- ARBITRES
-- ---------------------------------------------------------------------
CREATE TABLE arbitres(
    idArbitre INTEGER NOT NULL,
    idPersona INTEGER NOT NULL,
    colegi VARCHAR2(100),
    dataInici DATE NOT NULL,
    dataFi DATE,
    estat CHAR(1) DEFAULT 'A' NOT NULL
);
CREATE UNIQUE INDEX p_arbitres ON arbitres(idArbitre);
ALTER TABLE arbitres ADD CONSTRAINT p_arbitres PRIMARY KEY (idArbitre);

CREATE INDEX f_arbitres1 ON arbitres(idPersona);
ALTER TABLE arbitres ADD CONSTRAINT f_arbitres1 FOREIGN KEY (idPersona) REFERENCES persones(idPersona);

ALTER TABLE arbitres ADD CONSTRAINT c_arbitres1 CHECK (dataInici <= dataFi);
ALTER TABLE arbitres ADD CONSTRAINT c_arbitres2 CHECK (estat IN ('A', 'B'));

CREATE SEQUENCE s_arbitres1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER arbitres_ins BEFORE INSERT ON arbitres
    FOR EACH ROW
    BEGIN
        SELECT s_arbitres1.NEXTVAL INTO :NEW.idArbitre FROM DUAL;
    END;
/
CREATE OR REPLACE TRIGGER arbitres_upd BEFORE UPDATE ON arbitres
    FOR EACH ROW
    BEGIN
        IF (:NEW.estat != :OLD.estat AND :NEW.estat = 'B') THEN
            :NEW.dataFI := SYSDATE;
        END IF;
        IF (:NEW.estat != :OLD.estat AND :NEW.estat = 'A') THEN
            :NEW.dataFI := NULL;
        END IF;
    END;
/
-- ---------------------------------------------------------------------
-- TECNICS
-- ---------------------------------------------------------------------
CREATE TABLE tecnics(
    idTecnic INTEGER NOT NULL,
    idPersona INTEGER NOT NULL,
    dataInici DATE NOT NULL,
    dataFi DATE,
    estat CHAR(1) DEFAULT 'A' NOT NULL
);
CREATE UNIQUE INDEX p_tecnics ON tecnics(idTecnic);
ALTER TABLE tecnics ADD CONSTRAINT p_tecnics PRIMARY KEY (idTecnic);

CREATE INDEX f_tecnics1 ON tecnics(idPersona);
ALTER TABLE tecnics ADD CONSTRAINT f_tecnics1 FOREIGN KEY (idPersona) REFERENCES persones(idPersona);

ALTER TABLE tecnics ADD CONSTRAINT c_tecnics1 CHECK (dataInici <= dataFi);
ALTER TABLE tecnics ADD CONSTRAINT c_tecnics2 CHECK (estat IN ('A', 'B'));

CREATE SEQUENCE s_tecnics1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER tecnics_ins BEFORE INSERT ON tecnics
    FOR EACH ROW
    BEGIN
        SELECT s_tecnics1.NEXTVAL INTO :NEW.idTecnic FROM DUAL;
    END;
/
CREATE OR REPLACE TRIGGER tecnics_upd BEFORE UPDATE ON tecnics
    FOR EACH ROW
    BEGIN
        IF (:NEW.estat != :OLD.estat AND :NEW.estat = 'B') THEN
            :NEW.dataFI := SYSDATE;
        END IF;
        IF (:NEW.estat != :OLD.estat AND :NEW.estat = 'A') THEN
            :NEW.dataFI := NULL;
        END IF;
    END;
/
-- ---------------------------------------------------------------------
-- EQUIPS
-- ---------------------------------------------------------------------
CREATE TABLE equips(
    idEquip INTEGER NOT NULL,
    idRepresentant INTEGER NOT NULL,
    idMunicipi INTEGER NOT NULL,
    nom VARCHAR(100) NOT NULL,
    adreca VARCHAR2(100) NOT NULL,
    telefon VARCHAR2(50) NOT NULL,
    urlWeb VARCHAR2(250),
    tipus CHAR(2) NOT NULL,
    numSocis INTEGER DEFAULT 0 NOT NULL,
    estat CHAR(1) DEFAULT 'A' NOT NULL
);
CREATE UNIQUE INDEX p_equips ON equips(idEquip);
ALTER TABLE equips ADD CONSTRAINT p_equips PRIMARY KEY (idEquip);

CREATE INDEX f_equips1 ON equips(idRepresentant);
ALTER TABLE equips ADD CONSTRAINT f_equips1 FOREIGN KEY (idRepresentant) REFERENCES representantsLegals(idRepresentant);

CREATE INDEX f_equips2 ON equips(idMunicipi);
ALTER TABLE equips ADD CONSTRAINT f_equips2 FOREIGN KEY (idMunicipi) REFERENCES municipis(idMunicipi);

ALTER TABLE equips ADD CONSTRAINT c_equips1 CHECK (estat IN ('A', 'B'));

CREATE SEQUENCE s_equips1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER equips_ins BEFORE INSERT ON equips
    FOR EACH ROW
    BEGIN
        SELECT s_equips1.NEXTVAL INTO :NEW.idEquip FROM DUAL;
    END;
/
-- ---------------------------------------------------------------------
-- CONTRACTES
-- ---------------------------------------------------------------------
CREATE TABLE contractes(
    idContracte INTEGER NOT NULL,
    equipComprador INTEGER NOT NULL,
    equipVenedor INTEGER,
    idJugador INTEGER NOT NULL,
    dataInici DATE NOT NULL,
    dataFi DATE NOT NULL,
    salari NUMBER(16,2) NOT NULL,
    compensacio NUMBER(16,2) DEFAULT 0 NOT NULL,
    valorEconomic NUMBER(16,2) DEFAULT 0 NOT NULL,
    moneda CHAR(3) NOT NULL
);
CREATE UNIQUE INDEX p_contractes ON contractes(idContracte);
ALTER TABLE contractes ADD CONSTRAINT p_contractes PRIMARY KEY (idContracte);

CREATE INDEX f_contractes1 ON contractes(equipComprador);
ALTER TABLE contractes ADD CONSTRAINT f_contractes1 FOREIGN KEY (equipComprador) REFERENCES equips(idEquip);

CREATE INDEX f_contractes2 ON contractes(equipVenedor);
ALTER TABLE contractes ADD CONSTRAINT f_contractes2 FOREIGN KEY (equipVenedor) REFERENCES equips(idEquip);

CREATE INDEX f_contractes3 ON contractes(idJugador);
ALTER TABLE contractes ADD CONSTRAINT f_contractes3 FOREIGN KEY (idJugador) REFERENCES jugadors(idJugador);

ALTER TABLE contractes ADD CONSTRAINT c_contractes1 CHECK (dataInici <= dataFi);

CREATE SEQUENCE s_contractes1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER contractes_ins BEFORE INSERT ON contractes
    FOR EACH ROW
    BEGIN
        SELECT s_contractes1.NEXTVAL INTO :NEW.idContracte FROM DUAL;
    END;
/
-- ---------------------------------------------------------------------
-- COMPETICIONS
-- ---------------------------------------------------------------------
CREATE TABLE competicions(
    idCompeticio INTEGER NOT NULL,
    idPais CHAR(3) NOT NULL,
    nom VARCHAR2(50) NOT NULL,
    ambit CHAR(1) NOT NULL,
    genere char(1) NOT NULL,
    modalitat CHAR(1) NOT NULL,
    numArbitres NUMBER(5) NOT NULL,
    categoria CHAR(1) NOT NULL,
    temporada varchar2(50) NOT NULL,
    anyCompeticio NUMBER(5) NOT NULL,
    factor NUMBER(5,2) DEFAULT 1 NOT NULL
);
CREATE UNIQUE INDEX p_competicions ON competicions(idCompeticio);
ALTER TABLE competicions ADD CONSTRAINT p_competicions PRIMARY KEY (idCompeticio);

CREATE INDEX f_competicions1 ON competicions(idPais);
ALTER TABLE competicions ADD CONSTRAINT f_competicions1 FOREIGN KEY (idPais) REFERENCES paisos(idPais);

ALTER TABLE competicions ADD CONSTRAINT c_competicions1 CHECK (ambit IN ('N','C'));
ALTER TABLE competicions ADD CONSTRAINT c_competicions2 CHECK (genere IN ('M','F'));
ALTER TABLE competicions ADD CONSTRAINT c_competicions3 CHECK (modalitat IN ('N','C'));

CREATE SEQUENCE s_competicions1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER competicions_ins BEFORE INSERT ON competicions
    FOR EACH ROW
    BEGIN
        SELECT s_competicions1.NEXTVAL INTO :NEW.idCompeticio FROM DUAL;
    END;
/
-- ---------------------------------------------------------------------
-- PARTITS
-- ---------------------------------------------------------------------
CREATE TABLE partits(
    idPartit INTEGER NOT NULL,
    idCompeticio INTEGER NOT NULL,
    equipLocal INTEGER NOT NULL,
    equipVisitant INTEGER NOT NULL,
    puntsLocal INTEGER DEFAULT 0 NOT NULL,
    puntsVisitant INTEGER DEFAULT 0 NOT NULL,
    data DATE NOT NULL,
    hora char(5) NOT NULL,
    lloc varchar(100) NOT NULL
);
CREATE UNIQUE INDEX p_partits ON partits(idPartit);
ALTER TABLE partits ADD CONSTRAINT p_partits PRIMARY KEY (idPartit);

CREATE INDEX f_partits1 ON partits(idCompeticio);
ALTER TABLE partits ADD CONSTRAINT f_partits1 FOREIGN KEY (idCompeticio) REFERENCES competicions(idCompeticio);

CREATE INDEX f_partits2 ON partits(equipLocal);
ALTER TABLE partits ADD CONSTRAINT f_partits2 FOREIGN KEY (equipLocal) REFERENCES equips(idEquip);

CREATE INDEX f_partits3 ON partits(equipVisitant);
ALTER TABLE partits ADD CONSTRAINT f_partits3 FOREIGN KEY (equipVisitant) REFERENCES equips(idEquip);

CREATE SEQUENCE s_partits1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER partits_ins BEFORE INSERT ON partits
    FOR EACH ROW
    BEGIN
        SELECT s_partits1.NEXTVAL INTO :NEW.idPartit FROM DUAL;
    END;
/
-- ---------------------------------------------------------------------
-- TECNICSEQUIP
-- ---------------------------------------------------------------------
CREATE TABLE tecnicsEquip(
    idTecnic INTEGER NOT NULL,
    idEquip INTEGER NOT NULL,
    tipus CHAR(1) NOT NULL,
    dataInici DATE NOT NULL,
    dataFi DATE,
    estat CHAR(1) DEFAULT 'A' NOT NULL
);
CREATE UNIQUE INDEX p_tecnicsEquip ON tecnicsEquip(idTecnic);
ALTER TABLE tecnicsEquip ADD CONSTRAINT p_tecnicsEquip PRIMARY KEY (idTecnic);

CREATE INDEX f_tecnicsEquip1 ON tecnicsEquip(idEquip);
ALTER TABLE tecnicsEquip ADD CONSTRAINT f_tecnicsEquip1 FOREIGN KEY (idEquip) REFERENCES equips(idEquip) ON DELETE CASCADE;

ALTER TABLE tecnicsEquip ADD CONSTRAINT c_tecnicsEquip1 CHECK (tipus IN ('E','T','F'));
ALTER TABLE tecnicsEquip ADD CONSTRAINT c_tecnicsEquip2 CHECK (dataInici <= dataFi);
ALTER TABLE tecnicsEquip ADD CONSTRAINT c_tecnicsEquip3 CHECK (estat IN ('A', 'B'));

CREATE OR REPLACE TRIGGER tecnicsEquip_upd BEFORE UPDATE ON tecnicsEquip
    FOR EACH ROW
    BEGIN
        IF (:NEW.estat != :OLD.estat AND :NEW.estat = 'B') THEN
            :NEW.dataFI := SYSDATE;
        END IF;
        IF (:NEW.estat != :OLD.estat AND :NEW.estat = 'A') THEN
            :NEW.dataFI := NULL;
        END IF;
    END;
/
-- ---------------------------------------------------------------------
-- CONVOCATS
-- ---------------------------------------------------------------------
CREATE TABLE convocats(
    idPartit INTEGER NOT NULL,
    idJugador INTEGER NOT NULL,
    mins NUMBER(10) DEFAULT 0 NOT NULL,
    pt NUMBER(10) DEFAULT 0 NOT NULL,
    t1I NUMBER(10) DEFAULT 0 NOT NULL,
    t1E NUMBER(10) DEFAULT 0 NOT NULL,
    t2I NUMBER(10) DEFAULT 0 NOT NULL,
    t2E NUMBER(10) DEFAULT 0 NOT NULL,
    t3I NUMBER(10) DEFAULT 0 NOT NULL,
    t3E NUMBER(10) DEFAULT 0 NOT NULL,
    rd NUMBER(10) DEFAULT 0 NOT NULL,
    ro NUMBER(10) DEFAULT 0 NOT NULL,
    asi NUMBER(10) DEFAULT 0 NOT NULL,
    tf NUMBER(10) DEFAULT 0 NOT NULL,
    tc NUMBER(10) DEFAULT 0 NOT NULL,
    pr NUMBER(10) DEFAULT 0 NOT NULL,
    pp NUMBER(10) DEFAULT 0 NOT NULL,
    fc NUMBER(10) DEFAULT 0 NOT NULL,
    fr NUMBER(10) DEFAULT 0 NOT NULL,
    valoracio NUMBER(16) DEFAULT 0 NOT NULL,
    valoracioP NUMBER(16,2) DEFAULT 0 NOT NULL
);
CREATE UNIQUE INDEX p_convocats ON convocats(idPartit,idJugador);
ALTER TABLE convocats ADD CONSTRAINT p_convocats PRIMARY KEY (idPartit,idJugador);

CREATE INDEX f_convocats1 ON convocats(idPartit);
ALTER TABLE convocats ADD CONSTRAINT f_convocats1 FOREIGN KEY (idPartit) REFERENCES partits(idPartit);

CREATE INDEX f_convocats2 ON convocats(idJugador);
ALTER TABLE convocats ADD CONSTRAINT f_convocats2 FOREIGN KEY (idJugador) REFERENCES jugadors(idJugador) ON DELETE CASCADE;

-- ---------------------------------------------------------------------
-- SUBESTATS
-- ---------------------------------------------------------------------
CREATE TABLE subestats(
    idSubestat INTEGER NOT NULL,
    idJugador INTEGER NOT NULL,
    tipus CHAR(1) NOT NULL,
    descripcio VARCHAR2(500),
    dataDisponibilitat DATE NOT NULL,
    dataInici DATE NOT NULL,
    dataFiReal DATE
);
CREATE UNIQUE INDEX p_subestats ON subestats(idSubestat);
ALTER TABLE subestats ADD CONSTRAINT p_subestats PRIMARY KEY (idSubestat);

CREATE INDEX f_subestats1 ON subestats(idJugador);
ALTER TABLE subestats ADD CONSTRAINT f_subestats1 FOREIGN KEY (idJugador) REFERENCES jugadors(idJugador) ON DELETE CASCADE;

ALTER TABLE subestats ADD CONSTRAINT c_subestats1 CHECK (tipus IN ('P','M'));
ALTER TABLE subestats ADD CONSTRAINT c_subestats2 CHECK (dataInici <= dataFiReal);

CREATE SEQUENCE s_subestats1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER subestats_ins BEFORE INSERT ON subestats
    FOR EACH ROW
    BEGIN
        SELECT s_subestats1.NEXTVAL INTO :NEW.idSubestat FROM DUAL;
    END;
/

-- ---------------------------------------------------------------------
-- ARBITRATGES
-- ---------------------------------------------------------------------
CREATE TABLE arbitratges(
    idPartit INTEGER NOT NULL,
    idArbitre INTEGER NOT NULL,
    tipus CHAR(1) NOT NULL
);
CREATE UNIQUE INDEX p_arbitratges ON arbitratges(idPartit, idArbitre);
ALTER TABLE arbitratges ADD CONSTRAINT p_arbitratges PRIMARY KEY (idPartit, idArbitre);

CREATE INDEX f_arbitratges1 ON arbitratges(idPartit);
ALTER TABLE arbitratges ADD CONSTRAINT f_arbitratges1 FOREIGN KEY (idPartit) REFERENCES partits(idPartit) ON DELETE CASCADE;

CREATE INDEX f_arbitratges2 ON arbitratges(idArbitre);
ALTER TABLE arbitratges ADD CONSTRAINT f_arbitratges2 FOREIGN KEY (idArbitre) REFERENCES arbitres(idArbitre);

ALTER TABLE arbitratges ADD CONSTRAINT c_arbitratges1 CHECK (tipus IN ('P','A'));

-- ---------------------------------------------------------------------
-- LOGS
-- ---------------------------------------------------------------------
CREATE TABLE logs(
    idLog INTEGER NOT NULL,
    nomProc VARCHAR2(50) NOT NULL,
    data TIMESTAMP DEFAULT CURRENT_TIMESTAMP NOT NULL,
    paramsEntrada VARCHAR2(250),
    paramsSortida VARCHAR2(250),
    RSP VARCHAR2(250) NOT NULL
);
CREATE UNIQUE INDEX p_logs ON logs(idLog);
ALTER TABLE logs ADD CONSTRAINT p_logs PRIMARY KEY (idLog);

CREATE SEQUENCE s_logs1 INCREMENT BY 1 START WITH 1 NOCACHE;
CREATE OR REPLACE TRIGGER logs_ins BEFORE INSERT ON logs
    FOR EACH ROW
    BEGIN
        SELECT s_logs1.NEXTVAL INTO :NEW.idLog FROM DUAL;
    END;
/
-- =====================================================================
--  CREACIO DE FUNCIONS
-- =====================================================================
-- LLISTAT:
-- FUNCTION calcularPunts: Donat un nombre de tirs de 1, 2 i 3 punts calcula els punts resultants
-- FUNCTION calcularValoracio: Calcula la valoracio d'un jugador segons les estadistiques indicades
-- FUNCTION calcularValoracioP: Calcula la valoracio ponderada d'un jugador en un partit
-- TRIGGER convocats_ins: Disparador d'inserci de la taula convocats
-- TRIGGER convocats_upd: Disparador de modificaci de la taula convocats

-- ---------------------------------------------------------------------
-- NOM: calcularPunts
-- DESCRIPCIO: Donat un nombre de tirs de 1, 2 i 3 punts calcula els punts resultants
-- PARAMETRES ENTRADA: p_t1e - Tirs lliures (1 punt)
--                     p_t2e - Tirs de 2 (2 punts) 
--                     p_t3e - Triples (3 punts) 
-- PARAMETRES SORTIDA: punts - Punts totals
-- 
-- EXEMPLE: calcularPunts(5,3,2)
--          La sortida esperada es: 5*1 + 3*2 + 2*3 = 17 punts
-- ---------------------------------------------------------------------
CREATE OR REPLACE FUNCTION calcularPunts(
    p_t1e IN convocats.t1e%TYPE,
    p_t2e IN convocats.t2e%TYPE,
    p_t3e IN convocats.t3e%TYPE
) RETURN INTEGER
AS
BEGIN
    RETURN p_t1e + p_t2e * 2 + p_t3e * 3;
END;
/
-- ---------------------------------------------------------------------
-- NOM: calcularValoracio
-- DESCRIPCIO: Calcula la valoracio d'un jugador segons les estadistiques indicades
-- PARAMETRES ENTRADA: p_pt - Punts anotats
--                     p_t1e - Tirs lliure encistellats
--                     p_t2e - Tirs de 2 encistellats
--                     p_t3e - Triples encistellats
--                     p_rd - Rebots defensius
--                     p_ro - Rebots ofensius
--                     p_asi - Asistencies
--                     p_tf - Taps a favor
--                     p_pr - Pilotes recuperades 
--                     p_fr - Faltes rebudes
--                     p_t1i - Tirs lliures intentats
--                     p_t2i - Tirs de 2 intentats
--                     p_t3i - Triples intentats
--                     p_tc - Taps en contra
--                     p_pp - Pilotes perdudes
--                     p_fc - Faltes comeses
-- PARAMETRES SORTIDA: valoracio - Valoracio calculada
-- 
-- EXEMPLE: calcularValoracio(1,30)
--          La sortida esperada sera: 30 * 0.8 = 24 punts
-- ---------------------------------------------------------------------
CREATE OR REPLACE FUNCTION calcularValoracio(
    p_pt convocats.pt%TYPE,
    p_t1e convocats.t1e%TYPE,
    p_t2e convocats.t2e%TYPE,
    p_t3e convocats.t3e%TYPE,
    p_rd convocats.rd%TYPE,
    p_ro convocats.ro%TYPE,
    p_asi convocats.asi%TYPE,
    p_tf convocats.tf%TYPE,
    p_pr convocats.pr%TYPE,
    p_fr convocats.fr%TYPE,
    p_t1i convocats.t1i%TYPE,
    p_t2i convocats.t2i%TYPE,
    p_t3i convocats.t3i%TYPE,
    p_tc convocats.tc%TYPE,
    p_pp convocats.pp%TYPE,
    p_fc convocats.fc%TYPE
) RETURN INTEGER
AS
BEGIN
    RETURN p_pt + p_t1e + p_t2e + p_t3e + p_rd + p_ro + p_asi + p_tf +
           p_pr + p_fr - p_t1i - p_t2i - p_t3i - p_tc - p_pp - p_fc;
END;
/

-- ---------------------------------------------------------------------
-- NOM: calcularValoracioP
-- DESCRIPCIO: Calcula la valoracio ponderada d'un jugador en un partit
-- PARAMETRES ENTRADA: p_idPartit - ID del partit
--                     p_valoracio - valoracio sense ponderar del jugador
-- PARAMETRES SORTIDA: valoracioP - Valoracio ponderada 
-- 
-- EXEMPLE: calcularValoracioP(1,30)
--          Suposem que el factor de ponderaci de la competicio del partit indicat es 0.8
--          La sortida esperada sera: 30 * 0.8 = 24 punts
-- ---------------------------------------------------------------------
CREATE OR REPLACE FUNCTION calcularValoracioP(
    p_idPartit partits.idPartit%TYPE,
    p_valoracio convocats.valoracio%TYPE
) RETURN NUMBER
AS
    factor competicions.factor%TYPE;
BEGIN
    -- Determinar el factor de ponderacio de la competicio asociada al partit
    SELECT competicions.factor 
      INTO factor 
      FROM partits, competicions 
     WHERE partits.idCompeticio = competicions.idCompeticio 
       AND partits.idPartit = p_idPartit;

    RETURN p_valoracio * factor;

END;
/
-- ---------------------------------------------------------------------
-- NOM: convocats_ins
-- DESCRIPCIO: Calcula els punts i les valoracions
-- ---------------------------------------------------------------------
CREATE OR REPLACE TRIGGER convocats_ins BEFORE INSERT ON convocats
    FOR EACH ROW
    BEGIN
        SELECT calcularPunts(:NEW.t1e, :NEW.t2e, :NEW.t3e)
          INTO :NEW.pt
          FROM DUAL;

        SELECT calcularValoracio(:NEW.pt, :NEW.t1e, :NEW.t2e, :NEW.t3e, :NEW.rd , :NEW.ro, :NEW.asi, :NEW.tf, :NEW.pr,
                                 :NEW.fr, :NEW.t1i, :NEW.t2i, :NEW.t3i, :NEW.tc, :NEW.pp, :NEW.fc)
          INTO :NEW.valoracio
          FROM DUAL;

        SELECT calcularValoracioP(:NEW.idPartit, :NEW.valoracio)
          INTO :NEW.valoracioP
          FROM DUAL;
    END;
/
-- ---------------------------------------------------------------------
-- NOM: convocats_upd
-- DESCRIPCIO: Calcula els punts i les valoracions
-- ---------------------------------------------------------------------
CREATE OR REPLACE TRIGGER convocats_upd BEFORE UPDATE ON convocats
    FOR EACH ROW
    BEGIN
        SELECT calcularPunts(:NEW.t1e, :NEW.t2e, :NEW.t3e)
          INTO :NEW.pt
          FROM DUAL;

        SELECT calcularValoracio(:NEW.pt, :NEW.t1e, :NEW.t2e, :NEW.t3e, :NEW.rd , :NEW.ro, :NEW.asi, :NEW.tf, :NEW.pr,
                                 :NEW.fr, :NEW.t1i, :NEW.t2i, :NEW.t3i, :NEW.tc, :NEW.pp, :NEW.fc)
          INTO :NEW.valoracio
          FROM DUAL;

        SELECT calcularValoracioP(:NEW.idPartit, :NEW.valoracio)
          INTO :NEW.valoracioP
          FROM DUAL;
    END;
/
-- =====================================================================
--  MODUL ESTADISTIC
-- =====================================================================
-- ---------------------------------------------------------------------
-- dwh_totalJugadors
-- ---------------------------------------------------------------------
CREATE TABLE dwh_totalJugadors(
    total NUMBER NOT NULL
);
-- Cas inicial. 
-- Donada la naturalesa de la taula, fent aquest insert inicial, estaviem codi a implementar.
INSERT INTO dwh_totalJugadors (total) VALUES (0);

-- ---------------------------------------------------------------------
-- dwh_anotadorsCompeticio
-- ---------------------------------------------------------------------
CREATE TABLE dwh_anotadorsCompeticio(
    idCompeticio INTEGER NOT NULL,
    nomCompeticio VARCHAR2(50) NOT NULL,
    anyTemporada INTEGER NOT NULL,
    idJugador INTEGER NOT NULL,
    nom VARCHAR2(50) NOT NULL,
    cognoms VARCHAR2(50) NOT NULL,
    punts NUMBER(10) NOT NULL
);
CREATE UNIQUE INDEX p_dwh_anotadorsCompeticio ON dwh_anotadorsCompeticio(idCompeticio, idJugador, anyTemporada);
ALTER TABLE dwh_anotadorsCompeticio ADD CONSTRAINT p_dwh_anotadorsCompeticio PRIMARY KEY (idCompeticio, idJugador, anyTemporada);

CREATE INDEX f_dwh_anotadorsCompeticio1 ON dwh_anotadorsCompeticio(idCompeticio);
ALTER TABLE dwh_anotadorsCompeticio ADD CONSTRAINT f_dwh_anotadorsCompeticio1 FOREIGN KEY (idCompeticio) REFERENCES competicions(idCompeticio) ON DELETE CASCADE;

CREATE INDEX f_dwh_anotadorsCompeticio2 ON dwh_anotadorsCompeticio(idJugador);
ALTER TABLE dwh_anotadorsCompeticio ADD CONSTRAINT f_dwh_anotadorsCompeticio2 FOREIGN KEY (idJugador) REFERENCES jugadors(idJugador) ON DELETE CASCADE;

CREATE INDEX i_dwh_anotadorsCompeticio1 ON dwh_anotadorsCompeticio(idCompeticio, anyTemporada);

-- ---------------------------------------------------------------------
-- dwh_millorPagats
-- ---------------------------------------------------------------------
CREATE TABLE dwh_millorPagats(
    idJugador INTEGER NOT NULL,
    nom VARCHAR2(50) NOT NULL,
    cognoms VARCHAR2(50) NOT NULL,
    modalitat CHAR(1) NOT NULL,
    genere CHAR(1) NOT NULL,
    salari NUMBER(16,2) NOT NULL
);
CREATE UNIQUE INDEX p_dwh_millorPagats ON dwh_millorPagats(idJugador, modalitat, genere);
ALTER TABLE dwh_millorPagats ADD CONSTRAINT p_dwh_millorPagats PRIMARY KEY (idJugador, modalitat, genere);

CREATE INDEX f_dwh_millorPagats1 ON dwh_millorPagats(idJugador);
ALTER TABLE dwh_millorPagats ADD CONSTRAINT f_dwh_millorPagats1 FOREIGN KEY (idJugador) REFERENCES jugadors(idJugador) ON DELETE CASCADE;

CREATE INDEX i_dwh_millorPagats1 ON dwh_millorPagats(modalitat, genere);

-- ---------------------------------------------------------------------
-- dwh_totalGuanys
-- ---------------------------------------------------------------------
CREATE TABLE dwh_totalGuanys(
    idJugador INTEGER NOT NULL,
    nom VARCHAR2(50) NOT NULL,
    cognoms VARCHAR2(50) NOT NULL,
    guanys NUMBER(16,2) NOT NULL
);
CREATE UNIQUE INDEX p_dwh_totalGuanys ON dwh_totalGuanys(idJugador);
ALTER TABLE dwh_totalGuanys ADD CONSTRAINT p_dwh_totalGuanys PRIMARY KEY (idJugador);

ALTER TABLE dwh_totalGuanys ADD CONSTRAINT f_dwh_totalGuanys1 FOREIGN KEY (idJugador) REFERENCES jugadors(idJugador) ON DELETE CASCADE;

-- ---------------------------------------------------------------------
-- dwh_souMigJugadors
-- ---------------------------------------------------------------------
CREATE TABLE dwh_souMigJugadors(
    idPais CHAR(3) NOT NULL,
    nomPais VARCHAR2(100) NOT NULL,
    anyTemporada INTEGER NOT NULL,
    genere CHAR(1) NOT NULL,
    modalitat CHAR(1) NOT NULL,
    souMig NUMBER(16,2) NOT NULL
);
CREATE UNIQUE INDEX p_dwh_souMigJugadors ON dwh_souMigJugadors(idPais, anyTemporada, genere, modalitat);
ALTER TABLE dwh_souMigJugadors ADD CONSTRAINT p_dwh_souMigJugadors PRIMARY KEY (idPais, anyTemporada, genere, modalitat);

CREATE INDEX f_dwh_souMigJugadors1 ON dwh_souMigJugadors(idPais);
ALTER TABLE dwh_souMigJugadors ADD CONSTRAINT f_dwh_souMigJugadors1 FOREIGN KEY (idPais) REFERENCES paisos(idPais) ON DELETE CASCADE;

-- ---------------------------------------------------------------------
-- dwh_millorsEquips
-- ---------------------------------------------------------------------
CREATE TABLE dwh_millorsEquips(
    idCompeticio INTEGER NOT NULL,
    nomCompeticio VARCHAR2(50) NOT NULL,
    anyTemporada INTEGER NOT NULL,
    idEquip INTEGER NOT NULL,
    nomEquip VARCHAR2(100) NOT NULL,
    tipus CHAR(1) NOT NULL,
    punts INTEGER NOT NULL
);
CREATE UNIQUE INDEX p_dwh_millorsEquips ON dwh_millorsEquips(idCompeticio, idEquip, anyTemporada, tipus);
ALTER TABLE dwh_millorsEquips ADD CONSTRAINT p_dwh_millorsEquips PRIMARY KEY (idCompeticio, idEquip, anyTemporada ,tipus);

CREATE INDEX f_dwh_millorsEquips1 ON dwh_millorsEquips(idCompeticio);
ALTER TABLE dwh_millorsEquips ADD CONSTRAINT f_dwh_millorsEquips1 FOREIGN KEY (idCompeticio) REFERENCES competicions(idCompeticio) ON DELETE CASCADE;

CREATE INDEX f_dwh_millorsEquips2 ON dwh_millorsEquips(idEquip);
ALTER TABLE dwh_millorsEquips ADD CONSTRAINT f_dwh_millorsEquips2 FOREIGN KEY (idEquip) REFERENCES equips(idEquip) ON DELETE CASCADE;

-- ---------------------------------------------------------------------
-- dwh_millorJugador
-- ---------------------------------------------------------------------
CREATE TABLE dwh_millorJugador(
    genere CHAR(1) NOT NULL,
    modalitat CHAR(1) NOT NULL,
    idJugador INTEGER NOT NULL,
    nom VARCHAR2(50) NOT NULL,
    cognoms VARCHAR2(50) NOT NULL,
    anyTemporada INTEGER NOT NULL,
    valoracioP NUMBER(16,2) NOT NULL
);
CREATE UNIQUE INDEX p_dwh_millorJugador ON dwh_millorJugador(idJugador, anyTemporada, genere, modalitat);
ALTER TABLE dwh_millorJugador ADD CONSTRAINT p_dwh_millorJugador PRIMARY KEY (idJugador, anyTemporada, genere, modalitat);

CREATE INDEX f_dwh_millorJugador1 ON dwh_millorJugador(idJugador);
ALTER TABLE dwh_millorJugador ADD CONSTRAINT f_dwh_millorJugador1 FOREIGN KEY (idJugador) REFERENCES jugadors(idJugador) ON DELETE CASCADE;

CREATE INDEX i_dwh_millorJugador1 ON dwh_millorJugador(anyTemporada, genere, modalitat);

COMMIT;